#
# ================================================================
#
# Table definition for mod_benchmark module:
#
# ================================================================

# ----------------------------------------------------------------
#
# T_BENCHMARK_LOG: contains one row per HTTP request for which
#		   the module is activated
# ----------------------------------------------------------------
DROP TABLE IF EXISTS T_BENCHMARK_LOG;
CREATE TABLE T_BENCHMARK_LOG (
	LOG_DATE 	DATETIME NOT NULL DEFAULT "2002-01-01 00:00:00",
	URL_VALUE	VARCHAR(250),
	ARG_VALUE	VARCHAR(250),
	FIL_VALUE	VARCHAR(250),
	LOG_STATUS	INT NOT NULL DEFAULT 0,
	BEN_SEC		INT NOT NULL DEFAULT 0,
	BEN_USEC	INT NOT NULL DEFAULT 0,
	BEN_READY	INT NOT NULL DEFAULT 0,
	BEN_BUSY	INT NOT NULL DEFAULT 0,
	BEN_PZOMBIE	INT NOT NULL DEFAULT 0,
	BEN_PRUNNING	INT NOT NULL DEFAULT 0,
	BEN_PSTOPPED	INT NOT NULL DEFAULT 0,
	BEN_PSLEEPING	INT NOT NULL DEFAULT 0,
	BEN_AVGLOAD	INT NOT NULL DEFAULT 0,
	BEN_MEMUSE	INT NOT NULL DEFAULT 0,
	BEN_SWAPUSE	INT NOT NULL DEFAULT 0,
	BEN_CLIADDR	INT NOT NULL DEFAULT 0,
	BEN_SRVADDR	INT NOT NULL DEFAULT 0,
	BEN_SRVPORT	INT NOT NULL DEFAULT 0
	);

# Notes:
# =====
#	Request Stats:
#	-------------
#	LOG_STATUS: HTTP request status
#	BEN_SEC: time for response (in seconds)
#	BEN_USEC: microseconds part
#
#	Apache Stats:
#	------------
#	BEN_READY: count of ready process
#	BEN_BUSY: count of busy process
#
#	Process stats:
#	-------------
#	BEN_PZOMBIE: count of zombie process (unuseful ?)
#	BEN_PRUNNING: count of running process
#	BEN_PSTOPPED: count of stopped process
#	BEN_PSLEEPING: coutn of sleepin process
#
# 	The following percentages are given on a 1000 basis:
#
#	BEN_AVGLOAD: average CPU load
#	BEN_MEMUSE: percentage of memory used
#	BEN_SWAPUSE: percentage of swap used
#
#	Fields added from version 1.2:
#	-----------------------------
#	FIL_VALUE: filename after URI translation
#	BEN_CLIADDR: IP address of client
#	BEN_SRVADDR: IP address of server
#
#	Fields added from version 1.5:
#	-----------------------------
#	BEN_SRVPORT: local port (HTTP, HTTPS...)
#

# ----------------------------------------------------------------
#
# T_BENCHMARK_ALERT: contains one row per generated alert
#
# ----------------------------------------------------------------
DROP TABLE IF EXISTS T_BENCHMARK_ALERT;
CREATE TABLE T_BENCHMARK_ALERT (
	LOG_DATE 	DATETIME NOT NULL DEFAULT "2002-01-01 00:00:00",
	URL_VALUE	VARCHAR(250),
	ARG_VALUE	VARCHAR(250),
	FIL_VALUE	VARCHAR(250),
	BEN_SEC		INT NOT NULL DEFAULT 0,
	BEN_USEC	INT NOT NULL DEFAULT 0,
	BEN_CLIADDR	INT NOT NULL DEFAULT 0,
	BEN_SRVADDR	INT NOT NULL DEFAULT 0
	);

# Notes:
# =====
#
#	Fields added from version 1.2:
#	-----------------------------
#	FIL_VALUE: filename after URI translation
#	BEN_CLIADDR: IP address of client
#	BEN_SRVADDR: IP address of server
#

# ----------------------------------------------------------------
#
# T_BENCHMARK_GRAPH: contains the saved Reports
#
# ----------------------------------------------------------------
DROP TABLE IF EXISTS T_BENCHMARK_GRAPH;
CREATE TABLE T_BENCHMARK_GRAPH (
	GRA_ID		INT NOT NULL AUTO_INCREMENT PRIMARY KEY,
	GRA_NAME	VARCHAR(64) NOT NULL DEFAULT "",
	GRA_DATE	DATETIME NOT NULL DEFAULT "2002-01-01 00:00:00",
	GRA_NOTE	TEXT NOT NULL DEFAULT "",
	GRA_BLOB	TEXT NOT NULL DEFAULT "",
	GRA_WIDTH	INT NOT NULL DEFAULT 0,
	GRA_HEIGHT	INT NOT NULL DEFAULT 0
);

# Notes:
# =====
#
#	GRA_BLOB: Reports are limited to 64Kb
#

# EOF
