--
-- ================================================================
--
-- Table definition for mod_benchmark module:
--
-- ================================================================

-- ----------------------------------------------------------------
--
-- T_BENCHMARK_LOG: contains one row per HTTP request for which
--		   the module is activated
-- ----------------------------------------------------------------
CREATE TABLE T_BENCHMARK_LOG (
	LOG_DATE 	DATETIME,
	URL_VALUE	VARCHAR(250),
	ARG_VALUE	VARCHAR(250),
	FIL_VALUE	VARCHAR(250),
	LOG_STATUS	INT,
	BEN_SEC		INT,
	BEN_USEC	INT,
	BEN_READY	INT,
	BEN_BUSY	INT,
	BEN_PZOMBIE	INT,
	BEN_PRUNNING	INT,
	BEN_PSTOPPED	INT,
	BEN_PSLEEPING	INT,
	BEN_AVGLOAD	INT,
	BEN_MEMUSE	INT,
	BEN_SWAPUSE	INT,
	BEN_CLIADDR	INT,
	BEN_SRVADDR	INT,
	BEN_SRVPORT	INT
	);

-- Notes:
-- =====
--	Request Stats:
--	-------------
--	LOG_STATUS: HTTP request status
--	BEN_SEC: time for response (in seconds)
--	BEN_USEC: microseconds part
--
--	Apache Stats:
--	------------
--	BEN_READY: count of ready process
--	BEN_BUSY: count of busy process
--
--	Process stats:
--	-------------
--	BEN_PZOMBIE: count of zombie process (unuseful ?)
--	BEN_PRUNNING: count of running process
--	BEN_PSTOPPED: count of stopped process
--	BEN_PSLEEPING: coutn of sleepin process
--
-- 	The following percentages are given on a 1000 basis:
--
--	BEN_AVGLOAD: average CPU load
--	BEN_MEMUSE: percentage of memory used
--	BEN_SWAPUSE: percentage of swap used
--
--	Fields added from version 1.2:
--	-----------------------------
--	FIL_VALUE: filename after URI translation
--	BEN_CLIADDR: IP address of client
--	BEN_SRVADDR: IP address of server
--
--	Fields added from version 1.5:
--	-----------------------------
--	BEN_SRVPORT: local port (HTTP, HTTPS...)
--

-- ----------------------------------------------------------------
--
-- T_BENCHMARK_ALERT: contains one row per generated alert
--
-- ----------------------------------------------------------------
CREATE TABLE T_BENCHMARK_ALERT (
	LOG_DATE 	DATETIME,
	URL_VALUE	VARCHAR(250),
	ARG_VALUE	VARCHAR(250),
	FIL_VALUE	VARCHAR(250),
	BEN_SEC		INT,
	BEN_USEC	INT,
	BEN_CLIADDR	INT,
	BEN_SRVADDR	INT
	);

-- Notes:
-- =====
--
--	Fields added from version 1.2:
--	-----------------------------
--	FIL_VALUE: filename after URI translation
--	BEN_CLIADDR: IP address of client
--	BEN_SRVADDR: IP address of server
--

-- ----------------------------------------------------------------
--
-- T_BENCHMARK_GRAPH: contains the saved Reports
--
-- ----------------------------------------------------------------

-- Sequence added on the 22 Aug 2003:
CREATE SEQUENCE GRA_ID_SEQ START 1 INCREMENT 1;

CREATE TABLE T_BENCHMARK_GRAPH (
	GRA_ID		INT,
	GRA_NAME	VARCHAR(64),
	GRA_DATE	DATETIME,
	GRA_NOTE	TEXT,
	GRA_BLOB	TEXT,
	GRA_WIDTH	INT,
	GRA_HEIGHT	INT
);

-- Notes:
-- =====
--
--	GRA_BLOB: Reports are limited to 64Kb
--
-- EOF
