#!/usr/bin/perl -w
#
# email_message.inc.pl
#
use strict;
#use File::Basename;
use Digest::MD5;
use MIME::Base64;

package EmailMessage;

sub new
{
	my $class = shift;
	my $self = {};
	bless($self);

	# Private variables 
	$self->{'headers'} 	= ();
	$self->{'headers'}{"To"} = "";
	$self->{'headers'}{"Subject"} = "";
	$self->{'body'}   	= 1;
	$self->{'body_parts'}	= 0;
	$self->{'parts'}	= ();
	$self->{'delivery'}	= ();
	$self->{'delivery'}{"State"} = "";
	$self->{'next_token'} 	= "";

	# Public variables
	$self->{'email_regular_expression'}	= "^([-!#\$%&'*+./0-9=?A-Z^_`a-z{|}~])+@([-!#\$%&'*+/0-9=?A-Z^_`a-z{|}~]+\\.)+[a-zA-Z]{2,6}\$";
	$self->{'mailer'}			= "mod_benchmark";
	$self->{'default_charset'}		= "ISO-8859-1";
	$self->{'line_break'}			= "\n";
	$self->{'file_buffer_length'}		= 8000;
	$self->{'debug'}			= "1";
	$self->{'error'}			= "";


	bless($self,$class);
	return $self;
}

# Private methods
#sub Tokenize($string,$separator="")
sub Tokenize
{
	my ($self,$string,$separator) = @_;
	my $character;
	my $position;
	my $found;

	if (!defined($separator)) { $separator = ""; }

	if ($separator eq "")
	{
		$separator = $string;
		$string = $self->{'next_token'};
	}
	for ($character=0;$character<length($separator);$character++)
	{
		$position = index($string,substr($separator,$character,1));
		if ($position != -1) {
			if (defined($found)) { 
				if ($found > $position) { $found = $position; } 
			}
			else { $found = $position; }
		}
	}
	if(defined($found))
	{
		$self->{'next_token'} = substr($string,$found+1);
		return substr($string,0,$found);
	}

	$self->{'next_token'} = "";
	return $string;
}

sub GetFilenameExtension
{
	my ($self,$filename) = @_;
	my $dot;

	$dot = rindex($filename,".");
	if ($dot != -1) {
		return substr($filename,$dot);
	}

	return "";
}

sub OutputError
{
	my ($self,$error) = @_;

	$self->{'debug'} = "";
	#my $function = $self->{'debug'};
	#if ($function ne "" && $error ne "") {
	#	$function($error);
	#}

	print STDERR "$error\n";

	return $self->{'error'} = $error;
}

sub SendMail
{
	my ($self,$to,$subject,$body,$headers) = @_;	# headers,body = ref

	if ($self->{'debug'}) { print STDERR "SendMail($to,$subject,$headers)<br>\n"; }
		
	my $return_path = "";
	if (defined($self->{'delivery'}{'Headers'}))
	{
		my $header;
		my $key;
		while (($key,$header) = each %{$self->{'delivery'}{'Headers'}}) 
		{
			if ($key eq "Return-Path") 
			{
				$return_path = $header;
				last;
			}
		}
	}

	$return_path = "";
	my $success = mail($to,$subject,$body,$headers);
	return ($success ? "" : $self->OutputError("it was not possible to send e-mail message"));
}

sub StartSendingMessage
{
	my ($self) = @_;

	if ($self->{'debug'}) { print STDERR "StartSendingMessage()<br>\n"; }
		
	if ($self->{'delivery'}{'State'} ne "") {
		return $self->OutputError("the message was already started to be sent");
	}

	$self->{'delivery'}{"State"} = "SendingHeaders";

	return "";
}

sub SendMessageHeaders
{
	my ($self,$headers) = @_;

	if ($self->{'debug'}) { print STDERR "SendMessageHeaders($headers)<br>\n"; }
	
	if($self->{'delivery'}{'State'} ne "SendingHeaders")
	{
		if ($self->{'delivery'}{'State'} eq "") {
			return $self->OutputError("the message was not yet started to be sent");
		}
		else {
			return $self->OutputError("the message headers were already sent");
		}
	}

	$self->{'delivery'}{'Headers'} = $headers;
	$self->{'delivery'}{'State'}   = "SendingBody";

	return "";
}

sub SendMessageBody
{
	my ($self,$data) = @_;	# data=ref

	if ($self->{'debug'}) { print STDERR "SendMessageBody()<br>\n"; }
		
	if ($self->{'delivery'}{'State'} ne "SendingBody") {
		return $self->OutputError("the message headers were not yet sent");
	}

	if (defined($self->{'delivery'}{'Body'})) {
		$self->{'delivery'}{'Body'} .= $data;
	}
	else {
		$self->{'delivery'}{'Body'} = $data;
	}

	return "";
}

sub EndSendingMessage
{
	my ($self) = @_;

	if ($self->{'debug'}) { print STDERR "EndSendingMessage()<br>\n"; }
		
	if ($self->{'delivery'}{'State'} ne "SendingBody") {
		return $self->OutputError("the message body data was not yet sent");
	}

	if (!defined($self->{'delivery'}{'Headers'}) || count($self->{'delivery'}{'Headers'})==0) {
		return $self->OutputError("message has no headers");
	}

	my $line_break = $self->{'line_break'};
	my $headers_text;
	my $key;
	my $header;
	my $to;
	my $subject;
	while (($key,$header) = each %{$self->{'delivery'}{'Header'}}) 
	{
		if ($key eq "To") {
			$to = $header;
		}
		elsif ($key eq "Subject") {
			$subject = $header;
		}
		else {
			$headers_text .= "$key: $header.$line_break";
		}
	}

	if ($to eq "") {
		return $self->OutputError("it was not specified a valid To: header");
	}
	if ($subject eq "") {
		return $self->OutputError("it was not specified a valid Subject: header");
	}

	my $error = $self->SendMail($to,$subject,$self->{'delivery'}{'Body'},$headers_text);
	if ($error ne "") {
		return $error ;
	}
	$self->{'delivery'}{"State"} = "";

	return "";
}

sub StopSendingMessage
{
	my ($self) = @_;

	if ($self->{'debug'}) { print STDERR "StopSendingMessage()<br>\n"; }
		
	$self->{'delivery'}{"State"} = "";

	return "";
}

sub GetPartBoundary
{
	my ($self,$part);

	if ($self->{'debug'}) { print STDERR "GetPartBoundary($part)<br>\n"; }
		
	if (!defined($self->{'parts'}[$part]{'BOUNDARY'})) {
		$self->{'parts'}[$part]{'BOUNDARY'} = Digest::MD5::md5($part.time());
	}
}

sub GetPartHeaders
{
	my ($self,$headers,$part) = @_;		# headers = ref;

	if ($self->{'debug'}) { print STDERR "GetPartHeaders($part)<br>\n"; }
		
	if (!defined($self->{'parts'}[$part]{'Content-Type'})) {
		return $self->OutputError("it was added a part without Content-Type: defined");
	}

	#my $full_type = strtolower($self->{'parts'}[$part]{'Content-Type'};
	my $full_type = $self->{'parts'}[$part]{'Content-Type'};
	my $type      = $self->Tokenize($full_type,"/");
	my $sub_type  = $self->Tokenize("");

	if ($type eq "text" or $type eq "image" or $type eq "audio" or
	    $type eq "video" or $type eq "application") {
	    	my $text = "";
	    	if (defined($self->{'parts'}[$part]{'NAME'})) {
			$text = "; name=\"".$self->{'parts'}[$part]{'NAME'}."\"";
		}
		my $text2 = "";
		if (defined($self->{'parts'}[$part]{'CHARSET'})) {
			$text2 = "; charset=".$self->{'parts'}[$part]{'CHARSET'};
		}
		$headers->{'Content-Type'} = $full_type.$text2.$text;

		if (defined($self->{'parts'}[$part]{'Content-Transfer-Encoding'})) {
			$headers->{'Content-Transfer-Encoding'} = $self->{'parts'}[$part]{'Content-Transfer-Encoding'};
		}
	}
	elsif ($type eq "multipart") {
		if ($sub_type eq "alternative" or $sub_type eq "related" or
		    $sub_type eq "mixed" or $sub_type eq "parallel") {
			$self->GetPartBoundary($part);
			$headers->{"Content-Type"} = $full_type."; boundary=\"".$self->{'parts'}[$part]{"BOUNDARY"}."\"";
		}
		else {
			return $self->OutputError("multipart Content-Type sub_type $sub_type not yet supported");
		}
	}
	else {
		return $self->OutputError("Content-Type: $full_type not yet supported");
	}

	if (defined($self->{'parts'}[$part]{"Content-ID"})) {
		$headers->{"Content-ID"} = $self->{'parts'}[$part]{"Content-ID"};
	}

	return "";
}

sub GetPartBody
{
	my ($self,$body,$part) = @_;	# body = ref
	my $error;
	my $header;

	if ($self->{'debug'}) { print STDERR "GetPartBody($part)<br>\n"; }
		
	if (!defined($self->{'parts'}[$part]{"Content-Type"})) {
		return($self->OutputError("it was added a part without Content-Type: defined"));
	}

	#my $full_type = strtolower($self->parts[$part]{"Content-Type"});
	my $full_type = $self->{'parts'}[$part]{"Content-Type"};
	my $type      = $self->Tokenize($full_type,"/");
	my $sub_type  = $self->Tokenize("");

	if ($type eq "text" or $type eq "image" or $type eq "audio" or $type eq "video" or $type eq "application") {
		if (defined($self->{'parts'}[$part]{"FILENAME"}))
		{
			if (open(FD,"< $self->{'parts'}[$part]{'FILENAME'}") == 0) {
				return $self->OutputError("could not open part file");
			}

			my $block;
			while (sysread(FD,$block,8192)) 
			{
				${$body} .= $block;
			}
			close(FD);
		}
		else
		{
			if (!defined($self->{'parts'}[$part]{"DATA"})) {
				return$self->OutputError("it was added a part without a body PART");
			}
			${$body} = $self->{'parts'}[$part]{"DATA"};
		}

		my $encoding = "";
		if (defined($self->{'parts'}[$part]{'Content-Transfer-Encoding'})) {
			#$encoding = strtolower($self->parts[$part]{"Content-Transfer-Encoding"});
			$encoding = $self->{'parts'}[$part]{"Content-Transfer-Encoding"};
		}
		if ($encoding eq "base64") {
			${$body} = encode_base64(${$body});
		}
		elsif ($encoding eq "" or $encoding eq "quoted-printable") {
			# nothing
		}
		else {
			return $self->OutputError($encoding." is not yet a supported encoding type");
		}
	}
	elsif ($type eq "multipart") {
		if ($sub_type eq "alternative" or $sub_type eq "related" or $sub_type eq "mixed" or $sub_type eq "parallel") {
			$self->GetPartBoundary($part);
			my $boundary = $self->{'line_break'}."--".$self->{'parts'}[$part]{"BOUNDARY"};
			my $parts = scalar($self->{'parts'}[$part]{'PARTS'});
			my $multipart;

			print STDERR "parts=$parts\n";
			for($multipart = 0 ; $multipart < $parts ; $multipart++ )
			{
				${$body} .= $boundary.$self->{'line_break'};
				my %part_headers = ();
				my $sub_part = @{$self->{'parts'}[$part]{"PARTS"}}[$multipart];
				if (length($error=$self->GetPartHeaders(\%part_headers,$sub_part))) {
					return $error;
				}

				my $part_header;
				my $key;
				my $value;
				while (($key,$value) = each %part_headers) 
				{
					$header = $key;
					${$body}  .= "$key: $value"..$self->{'line_break'};
				}

				my $part_body;
				${$body} .= $self->{'line_break'};
				if (length($error = $self->GetPartBody(\$part_body,$sub_part))) {
					return $error;
				}
				${$body} .= $part_body;
			}
			${$body} .= $boundary."--".$self->{'line_break'};
		}
		else {
			return $self->OutputError("multipart Content-Type sub_type $sub_type not yet supported");
		}
	}
	else {
		return $self->OutputError("Content-Type: $full_type not yet supported");
	}

	return "";
}

# Public functions 
sub ValidateEmailAddress
{
	my ($self,$address) = @_;
	my $res = QuoteMeta($address);
	$res =~ $self->{'email_regular_expression'};

	return $res;
}

sub QuotedPrintableEncode
{
	my ($self,$text,$header_charset,$break_lines) = @_;	# header default = "", break default = 1
	my $index;
	my $code;

	if (!defined($header_charset)) { $header_charset = ""; }
	if (!defined($break_lines)) { $break_lines = 1; }

	if ($self->{'debug'}) { print STDERR "QuotedPrintableEncode()<br>\n"; }
		
	my $length = length($text);
	if($header_charset ne "")
	{
		$break_lines = 0;
		my @chars = split(//,$text);
		for ( $index = 0 ; $index < $length ; $index++ )
		{
			$code = ord($chars[$index]);
			if ($code < 32 || $code > 127) {
				last;
			}
		}
		if ($index > 0) {
			return(substr($text,0,$index).$self->QuotedPrintableEncode(substr($text,$index),$header_charset,0));
		}
	}

	my $encoded;
	my $encoded_length;
	my $character;
	my $line;
	my $order;
	my $previous_whitespace;
	my $whitespace;
	my $encode;
	my @chars = split(//,$text);

	for($whitespace=$encoded="",$line=0,$index=0;$index<$length;$index++)
	{
		$character = $chars[$index];
		$order     = ord($character);
		$encode    = 0;

		if ($order == 9 or $order == 32) {
			if ($header_charset eq "")
			{
				$previous_whitespace=$whitespace;
				$whitespace=$character;
				$character="";
			}
			else
			{
				if ($order == 32) { $character="_"; }
				else { $encode=1; }
			}
		}
		elsif ($order == 10 or $order == 13) {
			if ($whitespace ne "")
			{
				if ($break_lines && $line+3>75)
				{
					$encoded.="=".$self->{'line_break'};
					$line=0;
				}
				$encoded .=sprintf("=%02X",ord($whitespace));
				$line += 3;
				$whitespace = "";
			}
			$encoded.=$character;
			$line = 0;
			$index++;
			next;	# mimics "continue 2"
		}
		else {
			if ($order>127 || $order<32 || $character eq "=" || ($header_charset ne "" &&
			    ($character eq "?" || $character eq "_" || $character eq "(" || $character eq ")"))) {
				$encode=1;
			}
		}

		if ($whitespace ne "")
		{
			if ($break_lines && $line+1>75)
			{
				$encoded.="=".$self->{'line_break'};
				$line=0;
			}
			$encoded.=$whitespace;
			$line++;
			$whitespace="";
		}
		if($character ne "")
		{
			if($encode)
			{
				$character = sprintf("=%02X",$order);
				$encoded_length=3;
			}
			else {
				$encoded_length=1;
			}
			if($break_lines && $line+$encoded_length>75) 
			{
				$encoded.="=".$self->{'line_break'};
				$line=0;
			}
			$encoded.=$character;
			$line+=$encoded_length;
		}
	}
	if($whitespace ne "")
	{
		if($break_lines && $line+3>75) {
			$encoded.="=".$self->{'line_break'};
		}
		$encoded.=sprintf("=%02X",ord($whitespace));
	}
	if($header_charset ne "" && $text ne $encoded) {
		return "=?$header_charset?q?$encoded?=";
	}
	else {
		return $encoded;
	}
}

sub SetHeader
{
	my ($self,$header,$value,$encoding_charset) = @_;	# encoding default = ""

	if (!defined($encoding_charset)) { $encoding_charset = ""; }
	if ($self->{'debug'}) { print STDERR "SetHeader($header)<br>\n"; }
		
	if (length($self->{'error'})) {
		return($self->{'error'});
	}
	$self->{'headers'}{$header} = (($encoding_charset eq "") ? "$value" : $self->QuotedPrintableEncode($value,$encoding_charset));
	return "";
}

sub SetEncodedHeader
{
	my ($self,$header,$value) = @_;

	if ($self->{'debug'}) { print STDERR "SetEncodeHeader($header)<br>\n"; }
		
	return $self->SetHeader($header,$value,$self->{'default_charset'});
}

sub SetEncodedEmailHeader
{
	my ($self,$header,$address,$name) = @_;

	return $self->SetHeader($header,$address." (".$self->QuotedPrintableEncode($name,$self->{'default_charset'}).")");
}

sub CreatePart
{
	my ($self,$definition,$part) = @_;	# definition,part = ref;

	if ($self->{'debug'}) { print STDERR "CreatePart(".$definition->{"FILENAME"}.",part=".${$part}."<br>\n"; }
		
	${$part} = -1;
	if (length($self->{'error'})) { return($self->{'error'}); }

	if (!defined($self->{'parts'})) { # || scalar(@{$self->{'parts'}}) == 0) {
		${$part} = 0;
	}
	else {
		${$part} = scalar(@{$self->{'parts'}}) - 1;
	}

	$self->{'parts'}[${$part}] = $definition;

	return "";
}

sub AddPart
{
	my ($self,$part) = @_;
	my %parts; 
	my $error;
	my $body;

	if ($self->{'debug'}) { print STDERR "AddPart($part)<br>\n"; }
		
	if (length($self->{'error'})) {
		return($self->{'error'});
	}
	if ($self->{'body_parts'} == 0) {
		$self->{'body'} = $part;
	}
	elsif ($self->{'body_parts'} == 1) {
		%parts = (
				$self->{'body'},
				$part
			);
		if (length($error=$self->CreateMixedMultipart(\%parts,\$body))) {
			return($error);
		}
		$self->{'body'} = $body;
	}
	else {
		$self->{'parts'}[$self->{'body'}]{"PARTS"} = ([]);
		push(@{$self->{'parts'}[$self->{'body'}]{"PARTS"}},$part);
	}

	$self->{'body_parts'}++;
	return "";
}

sub CreateEncodedQuotedPrintableTextPart
{
	my ($self,$text,$charset,$part) = @_;	# part = ref

	if ($self->{'debug'}) { print STDERR "CreateEncodedQuotedPrintableTextPart($text)<br>\n"; }
		
	if($charset eq "") {
		$charset=$self->{'default_charset'};
	}
	my %definition= (
		"Content-Type"=>"text/plain",
		"Content-Transfer-Encoding"=>"quoted-printable",
		"CHARSET"=>$charset,
		"DATA"=>$text
	);
	return $self->CreatePart(\%definition,$part);
}

sub AddEncodedQuotedPrintableTextPart
{
	my ($self,$text,$charset) = @_;	# default charset = ""

	if (!defined($charset)) { $charset = ""; }

	if ($self->{'debug'}) { print STDERR "AddEncodedQuotedPrintableTextPart($text)<br>\n"; }
		
	my $error;
	my $part;
	if (length($error=$self->CreateEncodedQuotedPrintableTextPart($text,$charset,\$part)) || length($error=$self->AddPart($part))) {
		return($error);
	}
	return "";
}

sub AddQuotedPrintableTextPart
{
	my ($self,$text,$charset) = @_;	# default charset = ""

	if (!defined($charset)) { $charset = ""; }

	if ($self->{'debug'}) { print STDERR "AddQuotedPrintableTextPart($text)<br>\n"; }
		
	return $self->AddEncodedQuotedPrintableTextPart($self->QuotedPrintableEncode($text),$charset);
}

sub CreateFilePart
{
	my ($self,$file,$part) = @_;	# file,part = ref

	if ($self->{'debug'}) { print STDERR "CreateFilePart(".$file->{'FileName'}.")<br>\n"; }
		
	my $name = "";
	if (defined($file->{"FileName"})) {
		#$name = File::basename($file->{"FileName"});
		$name = $file->{"FileName"};
		$name =~ s(^.*/)();
	}
	else
	{
		if (!defined($file->{"Data"})) {
			return $self->OutputError("it was not specified the file part file name");
		}
	}
	if (defined($file->{'Name'})) {
		$name = $file->{"Name"};
	}
	if (length($name)==0) {
		return $self->OutputError("it was not specified the file part name");
	}

	my $content_type;
	my $type;
	my $sub_type;
	if (defined($file->{"Content-Type"}))
	{
		$content_type = $file->{"Content-Type"};
		#$type = $self->Tokenize(strtolower($content_type),"/");
		$type = $self->Tokenize($content_type,"/");
		$sub_type = $self->Tokenize("");

		if ($type eq "text" or $type eq "image" or $type eq "audio" or $type eq "video" or $type eq "application") {
			# nothing
		}
		else {
			return $self->OutputError($content_type." is not a supported file content type");
		}
	}
	else {
		$content_type="application/octet-stream";
	}

	my %definition = (
		"Content-Type"=>$content_type,
		"Content-Transfer-Encoding"=>"base64",
		"NAME"=>$name
	);
	if (defined($file->{"FileName"})) {
		$definition{"FILENAME"} = $file->{"FileName"};
	}
	else
	{
		if (defined($file->{"Data"})) {
			$definition{"DATA"} = $file->{"Data"};
		}
	}
	return $self->CreatePart(\%definition,$part);
}

sub CreateMultipart
{
	my ($self,$parts,$part,$type) = @_;	# parts,part = ref

	if ($self->{'debug'}) { print STDERR "CreateMultipart($type)<br>\n"; }
		
	my %definition = (
		"Content-Type"=>"multipart/".$type,
		"PARTS"=>$parts
	);
	return $self->CreatePart(\%definition,$part);
}

sub AddMultipart
{
	my ($self,$parts,$type) = @_;	# parts = ref

	if ($self->{'debug'}) { print STDERR "AddMultipart($type)<br>\n"; }
		
	my $error;
	my $part;
	if (length($error=$self->CreateMultipart($parts,\$part,$type)) || length($error=$self->AddPart($part))) {
		return($error);
	}

	return "";
}

sub CreateMixedMultipart
{
	my ($self,$parts,$part) = @_;	# parts,part = ref

	if ($self->{'debug'}) { print STDERR "CreateMixedMultipart($parts)<br>\n"; }
		
	return $self->CreateMultipart($parts,$part,"mixed");
}

sub AddMixedMultipart
{
	my ($self,$parts) = @_; 	# parts = ref

	if ($self->{'debug'}) { print STDERR "AddMixedMultipart($parts)<br>\n"; }
	
	return $self->AddMultipart($parts,"mixed");
}

sub Send
{
	my ($self) = @_;
	my $body;

	if ($self->{'debug'}) { print STDERR "Send()<br>\n"; }
		
	if (length($self->{'error'})) {
		return $self->{'error'};
	}

	my %headers = $self->{'headers'};
	if ($self->{'mailer'} ne "") {
		$headers{"X-Mailer"} = $self->{'mailer'};
	}
	$headers{"MIME-Version"} = "1.0";

	if ($self->{'body_parts'} == 0) {
		return $self->OutputError("message has no body parts");
	}

	my $error = $self->GetPartHeaders(\%headers,$self->{'body'});
	if (length($error)) {
		return($error);
	}
	if (($error=$self->StartSendingMessage()) ne "") {
		return($error);
	}
	if (length($error=$self->SendMessageHeaders(%headers))==0 && length($error=$self->GetPartBody(\$body,$self->{'body'}))==0 &&
	    length($error=$self->SendMessageBody($body))==0) {
		$error = $self->EndSendingMessage();
	}

	$self->StopSendingMessage();

	return $error;
}

1;
# EOF
