#!/usr/bin/perl -w
use strict;
#
# ====================================================================
#
# This is the script that displays the Real-time statistics.
#
# Date: 14 Aug 2003
# Author: Jerome Delamarche (c) Copyright INODES.  2003  All rights reserved.
# Version: 2.0
#
# Input Parameters:
#	width, height
#	duration: minute|hour|day|week|month|year
#	startdate: now|adate
#	startday, startmonth, startyear, starthour, startmin
#	viewtype: tabular|graph
#	graphtype: bar|line|pie|area
#	urls: all | url...
#	//servers: all | srv0 | srv1....
#	srv0, srv1 .... (multiple servers)
#	protocols: HTTP | HTTPS
#	responsetime,cpuload,memuse,swapuse,alertcount,apacheload,averagetime
#	threshold
#	avgval, maxval, minval
#
# Output: 
#	creates the $gImgName, $gTableOutput and the $gNavigationBar,
#	also creates the SVG files
#	output "statrtoutput.inc.pl" file
#
# --------------------------------------------------------------------
#
# TODO:
#
# ====================================================================
#
use POSIX;
use CGI;
use Time::Local;
use File::Basename;

my $query = CGI::new();

# -----------------------
# Initialize the Database:
# -----------------------
require "messages.inc.pl";
require "dbbenchmark.inc.pl";

my $db = CDBBenchmark->new($Globals::dbtype,$Globals::dbname,
				$Globals::dbserver,$Globals::dbuser,$Globals::dbpwd);


# -----------------------------------------------------
# Globalize the input parameters (except the DATA ones):
# -----------------------------------------------------
my $gDuration;
my $gViewType;
my $gStartDate;
my $gStartYear;
my $gStartMonth,
my $gStartDay;
my $gStartHour;
my $gStartMin;
my $gGraphType;
my $gThreshold;
my $gUrls;
my $gaServer;
my $gProtocols;
my $gAvgVal;
my $gMinVal;
my $gMaxVal;
my $gGraphWidth;
my $gGraphHeight;

my %allparms = ( 
			"duration" 	=> \$gDuration,
			"viewtype"	=> \$gViewType,
			"startdate"	=> \$gStartDate,
			"startyear"	=> \$gStartYear,
			"startmonth"	=> \$gStartMonth,
			"startday"	=> \$gStartDay,
			"starthour"	=> \$gStartHour,
			"startmin"	=> \$gStartMin,
			"graphtype"	=> \$gGraphType,
			"threshold"	=> \$gThreshold,
			"urls"		=> \$gUrls,
			"servers"	=> \$gaServer,
			"protocols"	=> \$gProtocols,
			"avgval"	=> \$gAvgVal,
			"minval"	=> \$gMinVal,
			"maxval"	=> \$gMaxVal,
			"width"		=> \$gGraphWidth,
			"height"	=> \$gGraphHeight,
		);

my $gHREFNavigationBar = "statrt.pl?";
my $gNavigationBar;
my $gTableOutput;
my @gServers;

my $key;
my $value;
while (($key,$value) = each %allparms) {
	${$value} = $query->param($key);

	# also computes the fixed part of the URL used for the navigation bar:
	if (index($key,"start") == 0) { next; }
	if (!defined($query->param($key))) { next; }

	$gHREFNavigationBar .= $key."=".${$value}."&";
}

if (!defined($gGraphType)) { $gGraphType = "polyline"; }

# Scan the params for the list of servers:
my $parm = $query->param("srv0");	# "all" server ?
if ((defined($parm) and $parm eq "yes") or (defined($gaServer) and $gaServer eq "srv0")) { push @gServers,"all"; }

my $idx;
my $addr;
my $srvlist = $db->GetServerList();
if ($srvlist) {
	for ( $idx = 0 ; $idx < scalar(@{$srvlist}) ; $idx++ ) {
		$parm = $query->param("srv".($idx+1));
		$addr = $srvlist->[$idx]{"BEN_SRVADDR"};
		if ($addr eq "0") { next; }
		if ((defined($parm) and $parm eq "yes") or (defined($gaServer) and $gaServer eq "srv".($idx+1))) {
			push(@gServers,$addr);
			$gHREFNavigationBar .= ($idx+1)."=yes";
		}
	}
}
#dieError("server count=".count($gServers)." ".$gServers[0]);
# ---------------------------
# Read the user's preferences:
# ---------------------------
#$gUserPref = $db->GetUserPrefs();
my $gImgType = "svg";

# ----------------
# Global variables:
# ----------------
my $gBgColor	   = "white";
my $gBarSpacing	   = 6;
my $gXLabel	   = "";
my $gDelta	   = 0;
my $gLowTimeLimit  = 0;
my $gHighTimeLimit = 0;
my %gImgName;

my @data;

# ----------------------------------------------------------
# Special case: no input parm means output the default image:
# ----------------------------------------------------------
if (!defined($gDuration) || !defined($gViewType)) {
	# empty call:
	$gTableOutput = " ";
	$gNavigationBar = "";
	$gImgName{"remote"} = "../images/dfltgraph.gif";
	&SendHTML();
	exit(0);
}

# -------------------------------------------------------
# Check we can display the stats with the required format:
# -------------------------------------------------------
if ($gViewType eq "graph") {
	if ($gImgType eq "svg") {
		require "svggraph.inc.pl";
	}
	else {
		&dieError("imgtype parameter is invalid (should be svg)");
	}
}
elsif ($gViewType eq "tabular") {
	#require "table.inc.pl";
}
else {
	&dieError("viewtype parameter is invalid (should be tabular or graph)");	
}

# --------------------------------
# Build the list of required stats:
# --------------------------------
require "statdesc.inc.pl";
my @gStatList;
my $v;
foreach $key (keys %StatDesc::gStats) {
	$v = $query->param($key);
	if ($v eq "yes") { push @gStatList,$key;}	
}
if (@gStatList == 0) {
	&dieError("No data specified...");
}

if (!defined($gAvgVal) && !defined($gMinVal) && !defined($gMaxVal)) {
	&dieError("No stat type specified...");
}

# ---------------
# Set the filters:
# ---------------
my %enddate;
my $dummy;
my $daycount;
if ($gStartDate eq "now") {
	($dummy,$enddate{'minutes'},$enddate{'hours'},$enddate{'mday'},$enddate{'mon'},$enddate{'year'}) = localtime(time);
	$enddate{'mon'}++;
	$enddate{'year'} += 1900;
}
else {
	$enddate{"year"} = $gStartYear;
	$enddate{"mon"}  = $gStartMonth;
	$enddate{"mday"} = $gStartDay;
	$enddate{"hours"} = $gStartHour;
	$enddate{"minutes"} = $gStartMin;	
}

# Adjust the enddate according to the duration:
my %curdate;
my $curtime;
$enddate{"seconds"} = 59;
if ($gDuration ne "minute") {
	$enddate{"minutes"} = 59;
	if ($gDuration ne "hour") {
		$enddate{"hours"} = 23;
		if ($gDuration eq "week") {
			# day must be next saturday
			$curtime = timelocal($enddate{'seconds'},$enddate{'minutes'},$enddate{'hours'},$enddate{'mday'},$enddate{'mon'}-1,$enddate{'year'}-1900);
			($dummy,$curdate{'minutes'},$curdate{'hours'},$curdate{'mday'},$curdate{'mon'},$curdate{'year'},$curdate{'wday'}) = localtime($curtime);
			$curdate{'mon'}++;
			$curdate{'year'} += 1900;

			$enddate{"mday"} += 6 - $curdate{"wday"};
			my $dmonth = $db->DayPerMonth($enddate{"mon"},$enddate{"year"});
			if ($enddate{"mday"} > $dmonth) {
				$enddate{"mon"}++;
				$enddate{"mday"} -= $dmonth;
				if ($enddate{"mon"} > 12) {
					$enddate{"mon"} = 1;
					$enddate{"year"}++;
				}
			}
		}
		elsif ($gDuration eq "month") {
				# day must be last day of current month
				$daycount = $db->DayPerMonth($enddate{"mon"},$enddate{"year"});
				$enddate{"mday"} = $daycount;
		}
		elsif ($gDuration eq "year") {
			$enddate{"mon"} = 12;
			$enddate{"mday"} = 31;
		}
	}
}

# Make sure values are written with 2 digits:
if ($enddate{"mon"} < 10) { $enddate{"mon"} = "0".$enddate{"mon"}; }
if ($enddate{"mday"} < 10) { $enddate{"mday"} = "0".$enddate{"mday"}; }
if ($enddate{"hours"} < 10) { $enddate{"hours"} = "0".$enddate{"hours"}; }
if ($enddate{"minutes"} < 10) { $enddate{"minutes"} = "0".$enddate{"minutes"}; }
if ($enddate{"seconds"} < 10) { $enddate{"seconds"} = "0".$enddate{"seconds"}; }

# Set the end date filter:
$db->SetEndDate($enddate{"year"}."-".$enddate{"mon"}."-".$enddate{"mday"}." ".$enddate{"hours"}.":".$enddate{"minutes"}.":".$enddate{"seconds"});
$gHighTimeLimit = timelocal($enddate{"seconds"},$enddate{"minutes"},$enddate{"hours"},$enddate{"mday"},$enddate{"mon"}-1,$enddate{"year"}-1900);

# Compute the start date:
if ($gDuration eq "minute") 	{	$gDelta = 60;			}
elsif ($gDuration eq "hour")	{	$gDelta = 60 * 60;		}
elsif ($gDuration eq "day")	{	$gDelta = 3600 * 24; 		}
elsif ($gDuration eq "week")	{	$gDelta = 3600 * 24 * 7;	}
elsif ($gDuration eq "month") 	{	$gDelta = 3600 * 24 * $daycount;	}
elsif ($gDuration eq "year") 	{	
	$gDelta = 3600 * 24 * 365;
	if (($enddate{"year"} % 4) == 0 && ($enddate{"year"} % 100) != 0) { $gDelta += 3600 * 24; }
}

$gDelta--;
$gLowTimeLimit = $gHighTimeLimit - $gDelta;
my %startdate;
($startdate{'seconds'},$startdate{'minutes'},$startdate{'hours'},$startdate{'mday'},$startdate{'mon'},$startdate{'year'},$startdate{'wday'}) = localtime($gLowTimeLimit);
$startdate{'mon'}++;
$startdate{'year'} += 1900;
if ($startdate{"mon"} < 10) { $startdate{"mon"} = "0".$startdate{"mon"}; }
if ($startdate{"mday"} < 10) { $startdate{"mday"} = "0".$startdate{"mday"}; }
if ($startdate{"hours"} < 10) { $startdate{"hours"} = "0".$startdate{"hours"}; }
if ($startdate{"minutes"} < 10) { $startdate{"minutes"} = "0".$startdate{"minutes"}; }
if ($startdate{"seconds"} < 10) { $startdate{"seconds"} = "0".$startdate{"seconds"}; }

# Set the start date filter:
$db->SetStartDate($startdate{"year"}."-".$startdate{"mon"}."-".$startdate{"mday"}." ".$startdate{"hours"}.":".$startdate{"minutes"}.":".$startdate{"seconds"});

# Set other filters:
$db->SetThreshold($gThreshold);
$db->SetRequest($gUrls);
$db->SetProtocols($gProtocols);

# ----------------------
# Compute the statistics:
# ----------------------
$gXLabel = &BuildXLabel(\%startdate,\%enddate,$gDuration);
my $gYScale = 1;
my @allres;
my $xcount  = 1;
my $xinterv = 10;
my $xstart  = 0;

if ($gDuration eq "minute") {
	$xcount = 60;	# 60 seconds
}
elsif ($gDuration eq "hour") {
	$xcount = 60;	# 60 minutes
}
elsif ($gDuration eq "day") {
	$xcount = 24;	# 24 jours
	$xinterv = 1;
}
elsif ($gDuration eq "week") {
	$xcount = 7;	# 7 days
	$xinterv = 1;
}
elsif ($gDuration eq "month") {
	$xcount = $db->DayPerMonth($startdate{"mon"},$startdate{"year"});
	$xinterv = 1;
	$xstart = 1;
}
elsif ($gDuration eq "year") {
	$xcount = 12;	# 12 months
	$xinterv = 1;
}
else {
	&dieError("Unknown scale ($gDuration)");
}

# Compute each required statistics: this requires three loops:
# one for the server, one for the stat and one for the stat type (max,min or avg)
my $srv;
my $s;
my $res;
for ( $srv = 0 ; $srv < scalar(@gServers) ; $srv++ ) {
	# Set the server filter:
	$db->SetServers($gServers[$srv]);
	
	for ( $s = 0 ; $s < scalar(@gStatList) ; $s++ ) {
		if ($gAvgVal) {
			$res = $db->GetStat($gStatList[$s],"avg",$gDuration);
			&MergeData(\@allres,$res,$gStatList[$s],"avg",$xcount);
		}
		if ($gMinVal) {
			$res = $db->GetStat($gStatList[$s],"min",$gDuration);
			&MergeData(\@allres,$res,$gStatList[$s],"min",$xcount);
		}
		if ($gMaxVal) {
			$res = $db->GetStat($gStatList[$s],"max",$gDuration);
			&MergeData(\@allres,$res,$gStatList[$s],"max",$xcount);
		}
	}
}

&DisplayAllData(\@allres,$xcount,$xinterv,$xstart);

# ----------------------------------------------------
# Compute the navigation bar (displyed under the graph:
# ----------------------------------------------------
$gNavigationBar = &ComputeNavigationBar();

# -------------------------
# Result output:
# -------------------------
&SendHTML();
exit(0);

# =========================================================================
# Internal Functions:
# =========================================================================

# Build the X Label from the duration and the interval of time:
sub BuildXLabel
{
	my ($sd,$ed,$dur) = @_;
	my $label;

	# always set the year, month and day:
	$label = "From: ".$sd->{"mday"}."-".&Messages::ShortMonth($sd->{"mon"})."-0".($sd->{"year"} % 10);
	if ($dur eq "hour" || $dur eq "minute") {
		$label .= " ".$sd->{"hours"}.":".$sd->{"minutes"}.":00";
	}
	elsif ($dur eq "day") { $label .= " 00:00:00"; }

	$label .= " To: ".$ed->{"mday"}."-".&Messages::ShortMonth($ed->{"mon"})."-0".($ed->{"year"} % 10);
	if ($dur eq "hour" || $dur eq "minute") {
		$label .= " ".$ed->{"hours"}.":".$ed->{"minutes"}.":59";
	}
	elsif ($dur eq "day") { $label .= " 23:59:59"; }

	return $label;
}

# Filter values to avoid Database errors:
sub Filter
{
	my ($v) = @_;

	if ($v < 0) { return 0; }
	return $v;
}

# ----------------------------
# Data merging functions:
# ----------------------------
sub MergeData
{
	my ($allres,$res,$stat,$stattype,$count) = @_;
	my $idx;

	my $grnu = scalar(@{$allres});
	for ( $idx = 0 ; $idx < $count ; $idx++ ) {
		$data[1]{$idx} = floor($res->[$idx] / $gYScale);
		$allres->[$grnu]{$idx} = &Filter($data[1]{$idx});
	}
	$allres->[$grnu]{"stat"} = $stat;
	$allres->[$grnu]{"stattype"} = $stattype;
}

# -----------------
# Display Functions:
# -----------------
sub DisplayAllData
{
	my ($allres,$xcount,$xinterv,$xstart) = @_;
	my $title;
	
	# X values:
	if ($gDuration eq "week") {
		$idx = 0;
		foreach $value (@{$Messages::gShortDay{'en'}}) {
			$data[0]{$idx++} = $value;
		}
		$data[0]{'xcount'} = 7;
	}
	elsif ($gDuration eq "year") {
		while (($key,$value) = each(%{$Messages::gShortMonth{"en"}})) {
			$data[0]{int($key-1)} = $value;
		}
		$data[0]{'xcount'} = 12;
	}
	else {
		for ( $idx = 0 ; $idx < $xcount ; $idx++ ) {
			if ((defined($gViewType) and $gViewType eq "graph") && (($idx % $xinterv) != 0)) {
				$data[0]{$idx} = "";
			}
			else {
				$data[0]{$idx} = $idx+$xstart;
			}
		}
		$data[0]{'xcount'} = $xcount;
	}

	# Y values:
	my $c;
	for ( $c = 0 ; $c < scalar(@{$allres}) ; $c++ ) {
		$data[1+$c]{"stat"} = $allres->[$c]{"stat"};
		$data[1+$c]{"stattype"} = $allres->[$c]{"stattype"};
		for ( $idx = 0 ; $idx < $xcount ; $idx++ ) {
			$data[1+$c]{$idx} = $allres->[$c]{$idx};
		}
		$data[1+$c]{'xcount'} = $xcount;
	}

	# TODO: $title not set ?
	$title = '';
	&CreateOutput($title);
}

# ================================
# Output functions:
# ================================
sub CreateOutput
{
	my ($title) = @_;
	#global $gViewType;
	
	if ($gViewType eq "tabular") { &CreateTable($title); }
	else { &CreateGraph($title); }
}

# --------------------------------
# Create a table with given values:
# --------------------------------
sub CreateTable
{
	my ($title) = @_;

	$gTableOutput .= "<p class=paramtitle align=center>".$gXLabel."</p>\n";
	$gTableOutput .= "<table border=1 width=100% cellpadding=2 cellspacing=0>";
	
	# there is one row per X value
	# there is one column per data. Stats are gather according 
	# to the data displayed
	my $statcount = scalar(@gStatList);
	my $stattypes = (scalar(@data) - 1) / $statcount;
	
	# count of rows for header:
	my $rowhdr = 2;
	if (scalar(@gServers) > 1) { $rowhdr++; }
	
	# Row with stat name:
	$gTableOutput .= "<tr>\n";
	$gTableOutput .= "<td width=40 rowspan=$rowhdr></td>\n";
	my $c;
	for ( $c = 0 ; $c < $statcount ; $c++ ) {
		$gTableOutput .= "<td class=textscale colspan=$stattypes>".$StatDesc::gStats{$gStatList[$c]}{"display"}."</td>\n";
	}
	$gTableOutput .= "</tr>\n";
	
	# For SRV stats, we must add row for each server:
	if (scalar(@gServers) > 1) {
		$gTableOutput .= "<tr>";
		my $srvcol = (scalar(@data) - 1) / scalar(@gServers);
		for ( $s = 0 ; $s < scalar(@gServers) ; $s++ ) {
			$gTableOutput .= "<td class=textstdblue align=center colspan=$srvcol>";
			if ($gServers[$s] eq "all") {
				$gTableOutput .= "all";
			}
			else {
				my $a = &Util::ntohl($gServers[$s]);
				$gTableOutput .= inet_ntoa(pack("L",$a));
			}
			$gTableOutput .= "</td>\n";
		}
		$gTableOutput .= "</tr>";
	}
	
	# Row with stat types:
	$gTableOutput .= "<tr>\n";
	for ( $c = 1 ; $c < scalar(@data) ; $c++ ) {
		$gTableOutput .= "<td class=textstdbold align=center>".$data[$c]{"stattype"}."</td>\n";
	}
	$gTableOutput .= "</tr>\n";
	
	my $r;
	for ( $r = 0 ; $r < $data[0]{'xcount'} ; $r++ ) {
		$gTableOutput .= "<tr>\n";
		$gTableOutput .= "<td class=textstdblue>".$data[0]{$r}."</td>\n";
		
		for ( $c = 1 ; $c < scalar(@data) ; $c++ ) {
			$gTableOutput .= "<td class=textstd align=center>".$data[$c]{$r}."</td>\n";
		}

		$gTableOutput .= "</tr>\n";
	}

	$gTableOutput .= "</table>\n";
}

# --------------------------------
# Create a Graph with given values:
# --------------------------------
sub CreateGraph
{
	my ($title) = @_;
	
	if ($gImgType eq "svg") {
		&CreateSVGGraph($title);
	}
		
	# other types not supported for now !
}

# --------------------------------
# Create a SVG Graph
# --------------------------------
sub CreateSVGGraph
{
	my ($title) = @_;
	my %legend;

	my $graph = CSVGGraph->new();
	$graph->{graphicWidth}      = $gGraphWidth;
	$graph->{graphicHeight}     = $gGraphHeight;
	$graph->{plotOffsetX}       = 150;
	$graph->{plotOffsetY}       = 20;
	$graph->{plotWidth}         = $graph->{graphicWidth} - $graph->{plotOffsetX} - 10;
	$graph->{plotHeight}        = $graph->{graphicHeight} - $graph->{plotOffsetY} - 20;
	$graph->{numGridLinesX}	    = 6;
	$graph->{numTicksX}	    = 10;
	$graph->{numGridlinesY}     = 11;
	$graph->{numTicksY}         = 1;
	$graph->{innerPaddingX}     = 10;
	$graph->{innerPaddingY}     = 10;
	$graph->{outerPaddingX}     = -60;
	$graph->{outerPaddingY}     = $graph->{plotOffsetX} - 50;
	$graph->{offsetGridlinesX}  = 0.1;
	$graph->{decimalPlacesY}    = 0;
	$graph->{rotTagsX}          = -30;
	$graph->{rotTagsY}          = 0;
	$graph->{minY}		    = 0;
	$graph->{maxY}		    = 100;	# minimum for maxY !
	$graph->{resolutionY}	    = 0;
 
	$graph->{title}             = $gXLabel;
	$graph->{outerPadding}	    = 0;
	$graph->{styleTitle}        = 'font-family: sans-serif; font-size: 8pt;';
	$graph->{labelX}            = ''; 	# $gXLabel;
	$graph->{styleLabelX}       = 'font-family: sans-serif; font-size: 6pt;';
	$graph->{labelY}            = "use in %    -    time in milli-sec";
	$graph->{styleLabelY}       = 'font-family: sans-serif; font-size: 8pt;';

	$graph->{dataX}		    = $data[0];
	
	# Legend Management:
	$graph->{legendPosX}	  = 4;
	$graph->{legendPosY}	  = 10;
	$graph->{legendTitle}       = "Legend";
	$graph->{legendStyle}	  = "font-family: sans-serif; font-size: 8pt;";
	$graph->{legendValueStyle}  = "font-family: sans-serif; font-size: 6pt;";
	$graph->{legendFilter}	  = ();
	
	if ($#gServers > 1) {
		$graph->{legendFilter}[0] = ();
		$graph->{legendFilter}[0]{"Filtername"} = $StatDesc::gStats{$data[1]{"stat"}}{"display"};
	}
	else {
		$graph->{legendFilter}[0]{"Filtername"} = "Server";
		if ($gServers[0] eq "all") {
			$graph->{legendFilter}[0]{'Value'} = "all";
		}
		else {
			my $a = &Util::ntohl($gServers[0]);
			$graph->{legendFilter}[0]{"Value"} = inet_ntoa(pack("L",$a));
		}
	}
	$graph->{legendFilter}[1]{"Filtername"} 	= "Protocol";
	$graph->{legendFilter}[1]{"Value"}      	= $gProtocols;
	$graph->{legendFilter}[2]{"Filtername"} 	= "URL";
	$graph->{legendFilter}[2]{"Value"}   		= substr($gUrls,0,16);
	
	# Associate a color and a shape to each data set:
	if (scalar(@gServers) > 1) {
		# Multiple server management:
		my $datapersrv = (scalar(@data) - 1) / scalar(@gServers);
		my $srvidx;
		my $srvcolor;
		for ( $srvidx = 0 ; $srvidx <= $#gServers ; $srvidx++ ) {
			for ( $idx = 1+($srvidx*$datapersrv) ; $idx < 1+(($srvidx+1)*$datapersrv) ; $idx++ ) {
				my $theStat = $data[$idx]{"stat"};
				$graph->{dataY}[$idx] = $data[$idx];
				#my @($graph->{format}[$idx]); 
		
				# stroke type depends on the stattype:
				my $stattype = $data[$idx]{"stattype"};
				my $stroketype = "";
				if ($stattype eq "min") 	{	$stroketype = "stroke-dasharray: 2,2;";	}
				elsif ($stattype eq "max") 	{	$stroketype = "stroke-dasharray: 6,2;";	}
		
				$srvcolor = $StatDesc::gServersColor[$srvidx % $#StatDesc::gServersColor];
				$graph->{format}[$idx]{"style"} = "stroke: $srvcolor; stroke-width:1; $stroketype filter:url(#dropShadow); shape-rendering: geometricPrecision; fill:none";
			}

			my $legnu = $#{$graph->{legendValue}}+1;
			if ($gServers[$srvidx] eq "all") {
				$graph->{legendValue}[$legnu]{'text'} = "all";
			}
			else {
				my $a = &Util::ntohl($gServers[$srvidx]);
				$graph->{legendValue}[$legnu]{"text"} = inet_ntoa(pack("L",$a));
			}
			$graph->{legendValue}[$legnu]{"style"} = "stroke: $srvcolor; stroke-width:2; shape-rendering: geometricPrecision; fill:none";
		}
	}
	else {
		# Multiple Data Management:
		for ( $idx = 1 ; $idx <= $#data ; $idx++ ) {
			my $theStat = $data[$idx]{"stat"};
			$graph->{dataY}[$idx] = $data[$idx];
			#$graph->format[$idx] = array();
		
			# stroke type depends on the stattype:
			my $stattype = $data[$idx]{"stattype"};
			my $stroketype = "";
			if ($stattype eq "min")			{	$stroketype = "stroke-dasharray: 2,2;";	}
			elsif ($stattype eq "max") 		{	$stroketype = "stroke-dasharray: 6,2;";	}
		
			$graph->{format}[$idx]{"style"} = "stroke:".$StatDesc::gStats{$theStat}{"color"}."; stroke-width:1; $stroketype filter:url(#dropShadow); shape-rendering: geometricPrecision; fill:none";
			#$graph->format[$idx]{"attributes"} = "marker-end='url(".$gStats{$theStat}{"shape"}.")'";
		
			if (!defined($legend{$theStat})) {
				$legend{$theStat} = "filled";
				my $legnu = $#{$graph->{legendValue}}+1;
				$graph->{legendValue}[$legnu]{"text"} = $StatDesc::gStats{$theStat}{"display"};
				$graph->{legendValue}[$legnu]{"style"} = "stroke:".$StatDesc::gStats{$theStat}{"color"}."; stroke-width:2; shape-rendering: geometricPrecision; fill:none";
			}
		}
	}
	
	$graph->{styleTagsX}    = 'font-family: sans-serif; font-size: 6pt;';
	$graph->{styleTagsY}    = 'font-family: sans-serif; font-size: 6pt;';
  	$graph->{styleGridX}	  = 'shape-rendering: crispEdges; stroke:#ccc; stroke-width:1; ';
  	$graph->{styleGridY}    = 'shape-rendering: crispEdges; stroke:#ccc; stroke-width:1; ';
  	$graph->{styleBox}      = 'shape-rendering: crispEdges; stroke:#000; stroke-width:1; fill:none;';

	# extra code for markers
	$graph->{extraSVG} = '
<filter id="dropShadow" filterUnits="objectBoundingBox" x="-10%" y="-10%" width="130%" height="130%">
  <feGaussianBlur in="SourceAlpha" stdDeviation="2" result="blur"/>
  <feOffset in="blur" dx="2" dy="2" result="offsetBlur"/>
  <feMerge>
    <feMergeNode in="offsetBlur"/>
    <feMergeNode in="SourceGraphic"/>
  </feMerge>
</filter>
';

	$graph->init()         or &dieError($graph->error);
	$graph->drawGraph();
	
	for ( $idx = 1 ; $idx <= $#data ; $idx++ ) {
		if ($gGraphType eq "bar") 	{	$graph->bar($idx)  or &dieError($graph->error);	}
		else 				{	$graph->polyline($idx)  or &dieError($graph->error);	}
	}

	$graph->generateSVG();

	my $curdir = dirname($ENV{'SCRIPT_FILENAME'});
	($gImgName{"local"},$gImgName{"remote"}) = &Util::GetImageName($curdir,"svg");
	open(FD,"> ".$gImgName{"local"});
	print FD $graph->{svg};
	close(FD);
}

# -------------------------------------------
# Compute the navigation bar:
# based on start/end date + duration value...
# -------------------------------------------
sub ComputeNavigationBar
{
	my $bar = "";
	my $href = $gHREFNavigationBar."startdate=adate&";

	# Do we have a "previous" interval ?
	my %d;
	my $t = $gLowTimeLimit;
	if ($t > 0) {
		($dummy,$d{'minutes'},$d{'hours'},$d{'mday'},$d{'mon'},$d{'year'},$d{'wday'}) = localtime($t - 1);
		$href .= "startyear=".($d{"year"}+1900)."&startmonth=".($d{"mon"}+1)."&startday=".$d{"mday"};
		$href .= "&starthour=".$d{"hours"}."&startmin=".$d{"minutes"};
		for ( $s = 0 ; $s < scalar(@gStatList) ; $s++ ) {
			$href .= "&".$gStatList[$s]."=yes";
		}
		$bar  .= "<a href=\"$href\"><img src=../images/prev.gif height=10 width=10 border=0></a> ";
	}

	$bar .= "1 ".$gDuration;
	$href = $gHREFNavigationBar."startdate=adate&";

	# Do we have a "next" interval ?
	$t = $gHighTimeLimit + 1;
	if ($t < time() + 1) {
		($dummy,$d{'minutes'},$d{'hours'},$d{'mday'},$d{'mon'},$d{'year'},$d{'wday'}) = localtime($t);
		$href .= "startyear=".($d{"year"}+1900)."&startmonth=".($d{"mon"}+1)."&startday=".$d{"mday"};
		$href .= "&starthour=".$d{"hours"}."&startmin=".$d{"minutes"};
		for ( $s = 0 ; $s < scalar(@gStatList) ; $s++ ) {
			$href .= "&".$gStatList[$s]."=yes";
		}
		$bar .= " <a href=\"$href\"><img src=../images/next.gif height=10 width=10 border=0></a>";
	}

	return $bar;
}

# --------------------------------------------
# Return the common HTML for all the statistic:
# --------------------------------------------
sub SendHTML
{
	# Clean the stats directory:
	my $curdir = dirname($ENV{'SCRIPT_FILENAME'});
	&Util::CleanReports($curdir,$Globals::gStatMaxAge);
	
	print $query->header;

print << "_END_HTML_";
<html>
<head>
	<title>Graph</title>
	<meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1">
	<link rel="stylesheet" type="text/css" href="../cpanel.css">
</head>

<body>
	<div align=center>
		<table width=100%>
			<tr>
				<td align=center>
_END_HTML_
			if (!defined($gDuration) or $gDuration eq "") {
				print "<img name=graph src=\"".$gImgName{'remote'}."\">\n";
			}
			if (defined($gViewType) and $gViewType eq "graph") { 
				if (defined($gImgType) and $gImgType eq "svg") {
					print "<embed name=graph type=\"image/svg\" ".
							"pluginspage=\"http://www.adobe.com/svg/viewer/install/\" ".
							"src=\"".$gImgName{"remote"}.
							"\" width=".$gGraphWidth.
							" height=".$gGraphHeight."></embed>\n";
				}
			}
			else {
				# view type is a table:
				print $gTableOutput."\n";
			}
print << "_END_HTML_";
				</td>
			</tr>
_END_HTML_
			if ($gNavigationBar ne "") {
print << "_END_HTML_";
			<tr>
				<td width=100% align=center>
					<span class=textstdblue>$gNavigationBar</span>
				</td>
			</tr>
_END_HTML_
			}
print << "_END_HTML_";
		</table>
	</div>
</body>					
</html>
_END_HTML_
}

sub dieError
{
	my ($text) = @_;

	print $query->header;

print << "_END_HTML_";
<html>
	<head>
		<link rel="stylesheet" type="text/css" href="../cpanel.css">
	</head>
	<body>
		<span class=textscale>
_END_HTML_
	print $text;
print << "_END_HTML_";
		</span>
	</body>
</html>
_END_HTML_
	die;
}

# EOF
