#!/usr/bin/perl -w
use strict;
#
# ====================================================================
#
# Small utility functions
#
# Date: 14 Aug 2003
# Author: Jerome Delamarche (c) Copyright INODES.  2003  All rights reserved.
# Version: 2.0
#
# --------------------------------------------------------------------
#
# TODO:
#
# ====================================================================
#
use POSIX;

require "globals.inc.pl";

package Util;

# 
# Utility functions:
#

# Test the browser type:
sub IsBrowserMSIE
{
	# TODO: passer par référence !
	my ($query) = @_;
	
	my $browser = $query->user_agent("MSIE");

	return ($browser ne "");
}

#
# ----------------------------------
# Generate the unique image filename:
# ----------------------------------
#
sub GetImageName
{
	my ($curdir,$suffix) = @_;
	my $local;
	my $remote;
	my $prefix;

	my $idx = 0;
	while ($idx++ < 10000) {
		srand(time);
		$prefix = int(rand(1000000));
		$local  = "$curdir/../stats/$prefix.$suffix";
		$remote = "../stats/$prefix.$suffix";

		if (! -e $local) { last; }
	}

	return ($local,$remote);
}

# -------------------------------
# Remove files older than max age:
# -------------------------------
sub CleanReports
{
	my ($curdir,$maxage) = @_;
	my $file;

	# $maxage is a count of hours:
	my $curtime = time();
	
	opendir(FD,"$curdir/../stats") or return;
	
	while (defined($file = readdir(FD))) {
		if ($file eq "." || $file eq "..") { next; }
			
		my ($mtime) = (stat("$curdir/../stats/$file"))[9];
		
		if ($curtime > $mtime + $maxage * 3600) {
			unlink("$curdir/../stats/$file");
		}
	}
	closedir(FD);
}

# Simple log function...
sub ErrorLog
{
	my ($text) = @_;

	syslog(3,$text);
}

# translate a TIMESTAMP into a DATETIME:
sub DateUnixToDB
{
	my ($d) = @_;
	my ($sec,$min,$hour,$mday,$mon,$year) = localtime($d);
	return POSIX::strftime("%Y-%m-%d %H:%M:00",$sec,$min,$hour,$mday,$mon,$year);
}

# Translate a DATETIME into a TIMESTAMP:
sub DateDBToUnix
{
	my ($d) = @_;

	my @p = split(/[- :]/,$d);

	return timelocal($p[5],$p[4],$p[3],$p[2],$p[1],$p[0]);
}

# Remove the "dangerous character" from a string before 
# an INSERT or an UPDATE SQL stmt:
sub SQLMakeStringSafe
{
	my ($s) = @_;

	$s =~ s/|\\"'//g;

	return $s;
}

# Remove the "dangerous character" from an integer before 
# an INSERT or an UPDATE SQL stmt:
sub SQLMakeIntSafe
{
	my ($s) = @_;

	$s =~ s/[^0-9]//g;

	return $s;
}

# Simple addslashes:
sub addslashes
{
	my ($s) = @_;

	$s =~ s/(["'])/\\$1/g;
	return $s;
}

# Simple stripslashes:
sub stripslashes
{
	my ($s) = @_;

	$s =~ s/\\(["'])/$1/g;
	return $s;
}

# Simple nothl():
sub ntohl
{
	my ($a) = @_;
	my $a1 = $a % 256;
	my $a2 = ($a >> 8) % 256;
	my $a3 = ($a >> 16 ) % 256;
	my $a4 = $a >> 24;

	return ((((($a1 << 8) + $a2) << 8) + $a3) << 8) + $a4;
}

1;
# EOF
