<?php
#
# ===========================================================================
#
# Class for Benchmark Table Management
#
# Date: 14 Aug 2003
# Author: Jerome Delamarche (c) Copyright INODES.  2003  All rights reserved.
# Version: 2.0
#
# ---------------------------------------------------------------------------
#
# TODO: an new function that contains the last DB error
#		
# ===========================================================================
#
include_once "util.inc.php";

# ===================================================
# All the database interface are defined as a 
# class with the following methods:
#
# XXX_connect($server,$user,$pwd,$db)
# XXX_QueryExecute($query)
# XXX_QueryAndFetchRows($query)
# XXX_QueryAndFetchaRow($query)
# XXX_LastError()
# XXX_SQLErrorLog($text)
#
# Since SQL language are different, we must
# implement some small methods that gives the exact
# SQL syntax for the WHERE clause:
#
# XXX_GetHourPart($field)
# XXX_GetMinutePart($field)
# XXX_GetSecondPart($field)
# XXX_GetDayOfWeek($field)
# XXX_GetWeek($field)
# XXX_GetDayOfMonth($field)
# XXX_GetMonth($field)
# XXX_GetYear($field)
# XXX_DateCompare($field,$date1,$date2)
# ===================================================

# ---------------------------------------------------
# MySQL Interface:
# ---------------------------------------------------
class CMySQLInterface
{
	var $m_conn = NULL;
	var $m_db   = NULL;

	function Connect($server,$user,$pwd,$dbname)
	{
		$this->m_conn = mysql_pconnect($server,$user,$pwd);
		if (!$this->m_conn) {
			$this->SQLErrorLog("Connection to $dbserver failed");
			die("Connexion to $dbserver failed: ".$php_errormsg);
		}

		if (!mysql_select_db($dbname,$this->m_conn)) {
			$this->SQLErrorLog("Database $dbname not found");
			die("Database not found: $dbname".$php_errormsg);
		}

		$this->m_db   = $dbname;
	}

	function QueryExecute($query)
	{
		mysql_select_db($this->m_db,$this->m_conn);
		$result = mysql_query($query,$this->m_conn);
		if (!$result) 
			return 0;
			
		return $result;
	}
	
	function QueryAndFetchRows($query)
	{
		$result = $this->Queryexecute($query);
		if (!$result) return 0;

		$count = 0;
		while ($row = mysql_fetch_array($result)) {
			$items[$count] = $row;
			$count++;
		}

		mysql_free_result($result);

		return $items;
	}

	function QueryAndFetchaRow($query)
	{
		$result = $this->Queryexecute($query);
		if (!$result) return 0;

		$row = mysql_fetch_array($result);

		mysql_free_result($result);

		return $row;
	}

	function LastError()
	{
		return mysql_error();
	}
		
	function SQLErrorLog($text)
	{
		ErrorLog("SQL Error: ".$text);
		ErrorLog("errno=".mysql_errno()." [".mysql_error()."]");
	}

	function GetHourPart($field) 	{ return "HOUR($field)"; }
	function GetMinutePart($field) 	{ return "MINUTE($field)"; }
	function GetSecondPart($field) 	{ return "SECOND($field)"; }
	function GetDayOfWeek($field)	{ return "DAYOFWEEK($field)"; }
	function GetWeek($field) 	{ return "WEEK($field)"; }
	function GetDayOfMonth($field)	{ return "DAYOFMONTH($field)"; }
	function GetMonth($field)	{ return "MONTH($field)"; }
	function GetYear($field)	{ return "YEAR($field)"; }

	function DateCompare($field,$date1,$date2) { return "$field > \"$date1\" AND $field <= \"$date2\""; }
};

# ---------------------------------------------------
# PostgreSQL Interface:
# ---------------------------------------------------
class CPostgreSQLInterface
{
	var $m_conn = NULL;
	var $m_db   = NULL;

	function Connect($server,$user,$pwd,$dbname)
	{
		$this->m_conn = pg_pconnect("host='$server' dbname='$dbname' user='$user' password='$pwd'");
		if (!$this->m_conn) {
			$this->SQLErrorLog("Connection to $dbserver failed");
			die("Connexion to $dbserver failed: ".$php_errormsg);
		}

		$this->m_db   = $dbname;
	}

	function QueryExecute($query)
	{
		$result = pg_query($this->m_conn,$query);
		if (!$result) 
			return 0;
			
		return $result;
	}
	
	function QueryAndFetchRows($query)
	{
		$result = $this->QueryExecute($query);
		if (!$result) return 0;

		$count = 0;
		while ($row = @pg_fetch_array($result,$count)) {
			// PostgreSQL PHP interface always returns column name
			// in lower case !
			while (list($key,$value) = each($row)) {
				$items[$count][strtoupper($key)] = $value;
			}
			$count++;
		}

		pg_free_result($result);

		return $items;
	}

	function QueryAndFetchaRow($query)
	{
		$result = $this->QueryExecute($query);
		if (!$result) return 0;

		$row = pg_fetch_array($result);

		// PostgreSQL PHP interface always returns column name
		// in lower case !
		while (list($key,$value) = each($row)) {
			$row2[strtoupper($key)] = $value;
		}

		pg_free_result($result);

		return $row2;
	}

	function LastError()
	{
		return pg_last_error();
	}
		
	function SQLErrorLog($text)
	{
		ErrorLog("SQL Error: ".$text);
		ErrorLog("error=".pg_last_error());
	}

	function GetHourPart($field) 	{ return "date_part('hour',$field)"; }
	function GetMinutePart($field) 	{ return "date_part('min',$field)"; }
	function GetSecondPart($field) 	{ return "date_part('sec',$field)"; }
	function GetDayOfWeek($field)	{ return "date_part('dow',$field)"; }
	function GetWeek($field) 	{ return "date_part('week',$field)"; }
	function GetDayOfMonth($field)	{ return "date_part('day',$field)"; }
	function GetMonth($field)	{ return "date_part('month',$field)"; }
	function GetYear($field)	{ return "date_part('year',$field)"; }

	function DateCompare($field,$date1,$date2) { return "$field > '$date1' AND $field <= '$date2'"; }
};

# ---------------------------------------------------
# Sybase Interface:
#
# Notes:
# Sybase extension sends a lot of WARNING through
# its callback mechanism. We must disable these
# messages...
# ---------------------------------------------------
class CSybaseInterface
{
	var $m_conn = NULL;
	var $m_db   = NULL;
	var $m_loglevel;

	function LowReporting()
	{
		return;
		$this->m_loglevel = error_reporting(0);
		error_reporting($this->m_loglevel & ~E_WARNING);
	}

	function StdReporting()
	{
		return;
		error_reporting($this->m_loglevel);
	}

	function Connect($server,$user,$pwd,$dbname)
	{
		$this->LowReporting();

		$this->m_conn = sybase_connect($server,$user,$pwd);
		if (!$this->m_conn) {
			$this->SQLErrorLog("Connection to $dbserver failed");
			die("Connexion to $dbserver failed: ".$php_errormsg);
		}

		if (!sybase_select_db($dbname,$this->m_conn)) {
			$this->SQLErrorLog("Database $dbname not found");
			die("Database not found: $dbname".$php_errormsg);
		}

		$this->m_db   = $dbname;

		$this->StdReporting();
	}

	function QueryExecute($query)
	{
		$this->LowReporting();
		$result = sybase_query($query,$this->m_conn);
		$this->StdReporting();
		if (!$result) 
			return 0;

		return $result;
	}
	
	function QueryAndFetchRows($query)
	{
		$result = $this->QueryExecute($query);
		if (!$result) 
			return 0;

		$this->LowReporting();
		$count = 0;
		while ($row = sybase_fetch_array($result)) {
			$items[$count] = $row;
			$count++;
		}

		sybase_free_result($result);
		$this->StdReporting();

		return $items;
	}

	function QueryAndFetchaRow($query)
	{
		$result = $this->QueryExecute($query);
		if (!$result) 
			return 0;

		$this->LowReporting();
		$row = sybase_fetch_array($result);
		sybase_free_result($result);
		$this->StdReporting();

		return $row;
	}

	function LastError()
	{
		return ""; // ????
	}
	
	function SQLErrorLog($text)
	{
		ErrorLog("SQL Error: ".$text);
	}

	function GetHourPart($field) 	{ return "datepart(hour,$field)"; }
	function GetMinutePart($field) 	{ return "datepart(minute,$field)"; }
	function GetSecondPart($field) 	{ return "datepart(second,$field)"; }
	function GetDayOfWeek($field)	{ return "datepart(weekday,$field)"; }
	function GetWeek($field) 	{ return "datepart(week,$field)"; }
	function GetDayOfMonth($field)	{ return "datepart(day,$field)"; }
	function GetMonth($field)	{ return "datepart(month,$field)"; }
	function GetYear($field)	{ return "datepart(year,$field)"; }

	# +++++ fixed on the 14 Aug 2003 (old version 1.6 did not care about HH:MM:SS)
	function DateCompare($field,$date1,$date2) 
	{ 
		list($y1,$m1,$d1,$h1,$i1,$s1) = split("[-: ]",$date1);
		$m1 = (int)$m1;
		list($y2,$m2,$d2,$h2,$i2,$s2) = split("[-: ]",$date2);
		$m2 = (int)$m2;
		return "$field > \"$m1/$d1/$y1 $h1:$i1:$s1\" AND $field <= \"$m2/$d2/$y2 $h2:$i2:$s2\""; 
	}
};

# ---------------------------------------------------
# Oracle Interface:
# ---------------------------------------------------
class COracleInterface
{
	var $m_conn = NULL;
	var $m_db   = NULL;
	var $m_loglevel;

	function Connect($server,$user,$pwd,$dbname)
	{
		$this->m_conn = ociplogon($user,$pwd,$server);
		if (!$this->m_conn) {
			$this->SQLErrorLog("Connection to $dbserver failed");
			$err = ocierror();
			die("Connexion to $dbserver failed: ".$err['message']);
		}

		$this->m_db   = $dbname;
	}

	function QueryExecute($query)
	{
		$result = ociparse($this->m_conn,$query);
		if (!$result) {
			$this->SQLErrorLog("ociparse failed for query: $query");
			return 0;
		}

		#print "$query<br>\n";
		$status = ociexecute($result,OCI_COMMIT_ON_SUCCESS);
		if (!$status) 
			return 0;

		return $result;
	}
	
	function QueryAndFetchRows($query)
	{
		#ociinternaldebug(1);
		$result = $this->QueryExecute($query);
		if (!$result) 
			return 0;

		$count = 0;
		while (ocifetchinto($result,&$row,OCI_ASSOC)) {
			$items[$count++] = $row;
		}

		ocifreestatement($result);

		return $items;
	}

	function QueryAndFetchaRow($query)
	{
		#ociinternaldebug(1);
		$result = $this->QueryExecute($query);
		if (!$result) 
			return 0;

		if (!ocifetchinto($result,&$row,OCI_ASSOC)) {
			$row['nothing'] = "";
		}

		ocifreestatement($result);

		return $row;
	}

	function LastError()
	{
		return ocierror(); 
	}
	
	function SQLErrorLog($text)
	{
		ErrorLog("SQL Error: ".$text);
	}

	function GetHourPart($field) 	{ return "TO_CHAR($field,'HH24')"; }
	function GetMinutePart($field) 	{ return "TO_CHAR($field,'MI')"; }
	function GetSecondPart($field) 	{ return "TO_CHAR($field,'SS')"; }
	function GetDayOfWeek($field)	{ return "TO_CHAR($field,'D')"; }
	function GetWeek($field) 	{ return "TO_CHAR($field,'WW')"; }
	function GetDayOfMonth($field)	{ return "TO_CHAR($field,'DD')"; }
	function GetMonth($field)	{ return "TO_CHAR($field,'MM')"; }
	function GetYear($field)	{ return "TO_CHAR($field,'YYYY')"; }

	function DateCompare($field,$date1,$date2) 
	{ 
		list($y1,$m1,$d1,$h1,$i1,$s1) = split("[-: ]",$date1);
		$m1 = (int)$m1;
		list($y2,$m2,$d2,$h2,$i2,$s2) = split("[-: ]",$date2);
		$m2 = (int)$m2;
		return "$field > TO_DATE('$m1/$d1/$y1 $h1:$i1:$s1','MM/DD/YYYY HH24:MI:SS') AND $field <= TO_DATE('$m2/$d2/$y2 $h2:$i2:$s2','MM/DD/YYYY HH24:MI:SS')"; 
	}
};

# ===================================================
# This is the main class that extracts data from 
# the benchmark database
# ===================================================
class CDBBenchmark
{
var $m_int 	= NULL;
var $m_db  	= NULL;
var $m_dbtype 	= NULL;

# filter values:
var $m_startdate = "2002-01-01 00:00:00";	// start date for stat computation
var $m_enddate   = "2010-12-31 23:59:59";	// end date
var $m_filterurl = "all";
var $m_threshold = 0;
var $m_servers   = "all";
var $m_protocols = "all";

# common queries:
var $m_avgQuery = Array (
		"apache" => "SELECT AVG(BEN_BUSY) AS VALUE FROM T_BENCHMARK_LOG",
		"cpu"	 => "SELECT AVG(BEN_AVGLOAD) AS VALUE FROM T_BENCHMARK_LOG",
		"mem"	 => "SELECT AVG(BEN_MEMUSE) AS VALUE FROM T_BENCHMARK_LOG",
		"swap"	 => "SELECT AVG(BEN_SWAPUSE) AS VALUE FROM T_BENCHMARK_LOG",
		"time"   => "SELECT AVG(BEN_SEC * 1000 + BEN_USEC / 1000) AS VALUE FROM T_BENCHMARK_LOG",
		"alert"  => "SELECT COUNT(*) AS VALUE FROM T_BENCHMARK_ALERT",
		);

var $m_maxQuery = Array (
		"apache" => "SELECT MAX(BEN_BUSY) AS VALUE FROM T_BENCHMARK_LOG",
		"cpu"	 => "SELECT MAX(BEN_AVGLOAD) AS VALUE FROM T_BENCHMARK_LOG",
		"mem"	 => "SELECT MAX(BEN_MEMUSE) AS VALUE FROM T_BENCHMARK_LOG",
		"swap"	 => "SELECT MAX(BEN_SWAPUSE) AS VALUE FROM T_BENCHMARK_LOG",
		"time"   => "SELECT MAX(BEN_SEC * 1000 + BEN_USEC / 1000) AS VALUE FROM T_BENCHMARK_LOG",
		"alert"  => "SELECT COUNT(*) AS VALUE FROM T_BENCHMARK_ALERT",
		);

var $m_minQuery = Array (
		"apache" => "SELECT MIN(BEN_BUSY) AS VALUE FROM T_BENCHMARK_LOG",
		"cpu"	 => "SELECT MIN(BEN_AVGLOAD) AS VALUE FROM T_BENCHMARK_LOG",
		"mem"	 => "SELECT MIN(BEN_MEMUSE) AS VALUE FROM T_BENCHMARK_LOG",
		"swap"	 => "SELECT MIN(BEN_SWAPUSE) AS VALUE FROM T_BENCHMARK_LOG",
		"time"   => "SELECT MIN(BEN_SEC * 1000 + BEN_USEC / 1000) AS VALUE FROM T_BENCHMARK_LOG",
		"alert"  => "SELECT COUNT(*) AS VALUE FROM T_BENCHMARK_ALERT",
		);

# ----------
# Ctor:
# ----------
function CDBBenchmark($dbtype, $dbname = "benchmark", $dbserver = NULL, $dbuser = NULL, $dbpwd = NULL) 
{
	$this->m_db     = $dbname;
	$this->m_dbtype = $dbtype;

	if ($dbtype == "MySQL")
		$this->m_int = new CMySQLInterface();
	elseif ($dbtype == "Sybase")
		$this->m_int = new CSybaseInterface();
	elseif ($dbtype == "PostgreSQL")
		$this->m_int = new CPostgreSQLInterface();
	elseif ($dbtype == "Oracle")
		$this->m_int = new COracleInterface();
	else die("Database '$dbtype' not supported\n");

	$this->m_int->Connect($dbserver,$dbuser,$dbpwd,$dbname);
}

# ---------------------
# Return the last error:
# ---------------------
function LastError()
{
	return $this->m_int->LastError();
}

# ---------------------------------------------------------
# Used by the menu to display the list of all available URL:
# ---------------------------------------------------------
function GetURLList()
{
	$query = "SELECT DISTINCT URL_VALUE FROM T_BENCHMARK_LOG ORDER BY URL_VALUE ASC";
	$result = $this->m_int->QueryAndFetchRows($query);
	if (!$result) {
		$this->m_int->SQLErrorLog($this->m_db.": CDBBenchmark::GetURLList() failed");
		return 0;
	}

	return $result;
}

# -------------------------------------------------------------
# Used by the menu to display the list of all available Servers:
# -------------------------------------------------------------
function GetServerList()
{
	$query = "SELECT DISTINCT BEN_SRVADDR FROM T_BENCHMARK_LOG ORDER BY BEN_SRVADDR ASC";
	$result = $this->m_int->QueryAndFetchRows($query);
	if (!$result) {
		$this->m_int->SQLErrorLog($this->m_db.": CDBBenchmark::GetServerList() failed");
		return 0;
	}

	return $result;
}

# ---------------------------------------------------------------
# Used by the menu to display the list of all available Protocols:
# ---------------------------------------------------------------
function GetProtocolList()
{
	$query = "SELECT DISTINCT BEN_SRVPORT FROM T_BENCHMARK_LOG ORDER BY BEN_SRVPORT ASC";
	$result = $this->m_int->QueryAndFetchRows($query);
	if (!$result) {
		$this->m_int->SQLErrorLog($this->m_db.": CDBBenchmark::GetProtocolList() failed");
		return 0;
	}

	return $result;
}

# =================================================
# Stored Graph Management:
# =================================================

# ----------------------------------------------
# Return the content of t_benchmark_graph table:
# ----------------------------------------------
function GetGraphList()
{
	$query = "SELECT * FROM T_BENCHMARK_GRAPH ORDER BY GRA_DATE DESC";
	$result = $this->m_int->QueryAndFetchRows($query);
	if (!$result) {
		$this->m_int->SQLErrorLog($this->m_db.": CDBBenchmark::GetGraphList() failed");
		return 0;
	}

	return $result;
}

# ----------------------------
# Retrieve the specified Graph:
# ----------------------------
function GraphGet($grid)
{
	$grid = SQLMakeIntSafe($grid);
	
	if ($this->m_dbtype != "Oracle") {
		$query = "SELECT * FROM T_BENCHMARK_GRAPH WHERE GRA_ID = $grid";
		$result = $this->m_int->QueryAndFetchaRow($query);
		if (!$result) {
			$this->m_int->SQLErrorLog($this->m_db.": CDBBenchmark::GraphGet($grid) failed");
			return 0;
		}
	}
	else {
		$query = "SELECT * FROM T_BENCHMARK_GRAPH WHERE GRA_ID = $grid";
		$stmt = ociparse($this->m_int->m_conn,$query);
		ociexecute($stmt);

		if (ocifetch($stmt)) {
			$result["GRA_NAME"] = ociresult($stmt,"GRA_NAME");
			$result["GRA_NOTE"] = ociresult($stmt,"GRA_NOTE");
			$result["GRA_DATE"] = ociresult($stmt,"GRA_DATE");
			$result["GRA_WIDTH"] = ociresult($stmt,"GRA_WIDTH");
			$result["GRA_HEIGHT"] = ociresult($stmt,"GRA_HEIGHT");
			$lob = ociresult($stmt,"GRA_BLOB");
			$result["GRA_BLOB"] = $lob->load();
		}
	}

	$result["GRA_NAME"] = stripslashes($result["GRA_NAME"]);
	$result["GRA_NOTE"] = stripslashes($result["GRA_NOTE"]);
	$result["GRA_BLOB"] = stripslashes($result["GRA_BLOB"]);

	if ($this->m_dbtype == "Oracle" ||
	    $this->m_dbtype == "Sybase") {
		$result["GRA_NAME"] = ereg_replace("''","'",$result["GRA_NAME"]);
		$result["GRA_NOTE"] = ereg_replace("''","'",$result["GRA_NOTE"]);
		$result["GRA_BLOB"] = ereg_replace("''","'",$result["GRA_BLOB"]);
	}
	
	return $result;
}

# --------------------------------
# Store a new graph:
# --------------------------------
function StoreGraph($name,$note,$graph,$w,$h)
{
	$w     = SQLMakeIntSafe($w);
	$h     = SQLMakeIntSafe($h);
	$name  = addslashes($name);
	$note  = addslashes($note);
	$graph = addslashes($graph);
	
	$date = DateUnixToDB(time());
	
	# patch for Oracle:
	if ($this->m_dbtype != "Oracle") {
		if ($this->m_dbtype == "Sybase") {
			$name  = ereg_replace("\\\'","''",$name);
			$note  = ereg_replace("\\\'","''",$note);
			$graph = ereg_replace("\\\'","''",$graph);
		}

		if ($this->m_dbtype == "PostgreSQL") {
			$query = "INSERT INTO T_BENCHMARK_GRAPH (GRA_ID,GRA_NAME,GRA_DATE,GRA_NOTE,GRA_BLOB,GRA_WIDTH,GRA_HEIGHT) ".
					"VALUES (NEXTVAL('GRA_ID_SEQ'),'$name','$date','$note','$graph',$w,$h)";
		}
		else {
			$query = "INSERT INTO T_BENCHMARK_GRAPH (GRA_NAME,GRA_DATE,GRA_NOTE,GRA_BLOB,GRA_WIDTH,GRA_HEIGHT) ".
					"VALUES ('$name','$date','$note','$graph',$w,$h)";
		}

		$res = $this->m_int->QueryExecute($query);
	}
	else  {
		$name  = ereg_replace("\\\'","''",$name);
		$note  = ereg_replace("\\\'","''",$note);
		$graph = ereg_replace("\\\'","''",$graph);
		$query = "INSERT INTO T_BENCHMARK_GRAPH (GRA_ID,GRA_NAME,GRA_DATE,GRA_NOTE,GRA_BLOB,GRA_WIDTH,GRA_HEIGHT) ".
				"VALUES (GRA_ID_SEQ.NEXTVAL,'$name',TO_DATE('$date','YYYY/MM/DD HH24:MI:SS'),'$note',EMPTY_CLOB(),$w,$h) ".
				"RETURNING GRA_BLOB INTO :THE_GRAPH";
		$stmt = ociparse($this->m_int->m_conn,$query);
		$lob = ocinewdescriptor($this->m_int->m_conn,OCI_D_LOB);
		ocibindbyname($stmt,":THE_GRAPH",&$lob,-1,OCI_B_CLOB);
		ociexecute($stmt,OCI_DEFAULT);	# lob operation caanot span transactions
		if ($lob->save($graph)) { ocicommit($this->m_int->m_conn); }
		ocifreedesc($lob);
		ocifreestatement($stmt);

		$res = 1;
	}
	
	return $res;
}

# --------------------------
# Delete the specified Graph:
# --------------------------
function GraphDelete($grid)
{
	$grid = SQLMakeIntSafe($grid);
	
	$query = "DELETE FROM T_BENCHMARK_GRAPH WHERE GRA_ID = $grid";

	$res = $this->m_int->QueryExecute($query);
	
	return $res;
}

#
# =================================================
# Function needed to compute statistics:
# =================================================
#
function SetStartDate($startdate) 	{ $this->m_startdate = $startdate; }
function SetEndDate($enddate) 		{ $this->m_enddate = $enddate; }
function SetRequest($req) 		{ $this->m_filterurl = $req; }
function SetServers($servers) 		{ 
	$this->m_servers = $servers;
	#$a = explode(".",$servers); $this->m_servers = ($a[0] << 24) + ($a[1] << 16) + ($a[2] << 8) + $a[3]; 
}
function SetProtocols($proto) 		{ $this->m_protocols = $proto; }
function SetThreshold($t) 			{ $this->m_threshold = $t; }

function CheckDateRange($year1,$month1,$day1,$year2,$month2,$day2)
{
	if ($year1 > $year2 ||
	    ($year1 == $year2 && $month1 > $month2) ||
	    ($year1 == $year2 && $month1 == $month2 && $day1 > $day2))
	    	return 0;
	
	return 1;
}

function NextDay($curday)
{
	list($year,$month,$day) = split("[- ]",$curday);
	$nextday = mktime(0,0,0,$month,$day,$year) + 3600 * 24;
	return strftime("%Y-%m-%d",$nextday);
}

# -------------------------------------------------
# Helper functions used to build the requests:
# -------------------------------------------------
function AddAvgFilter($stat)
{
	// always add the date constraint:
	$filter = " AND ".$this->m_int->DateCompare("LOG_DATE",$this->m_startdate,$this->m_enddate);

	// and now the optional filters:
	if ($this->m_threshold) 
		# fixed on the 30 July 2003
		#$threshold .= " AND (BEN_SEC * 1000 + ben_usec / 1000) > $this->m_threshold";
		$filter .= " AND (BEN_SEC * 1000 + BEN_USEC / 1000) > $this->m_threshold";

	if ($this->m_filterurl != "all" && !empty($this->m_filterurl))
		$filter .= " AND URL_VALUE = \"$this->m_filterurl\"";

	if ($this->m_servers != "all" && !empty($this->m_servers))
		# fixed on the 30 July 2003:
		#$filter .= " AND BEN_SRVADDR = ".ip2long($this->m_servers);
		$filter .= " AND BEN_SRVADDR = ".$this->m_servers;

	if ($stat != "alert" && $this->m_protocols != "all" && !empty($this->m_protocols))
		$filter .= " AND BEN_SRVPORT = ".$this->m_protocols;

	return $filter;
}

function AddMaxFilter($stat)
{
	return $this->AddAvgFilter($stat);
}

function AddMinFilter($stat)
{
	return $this->AddAvgFilter($stat);
}

# =================================================
# GENERIC MAX STATISTICS
# =================================================
function GetStat($stat,$stattype,$scale)
{
	# Get the query basis:
	if ($stattype == "max") {
		$query = $this->m_maxQuery[$stat];
		$filter = $this->AddMaxFilter($stat);
	}
	else if ($stattype == "avg") {
		$query = $this->m_avgQuery[$stat];
		$filter = $this->AddAvgFilter($stat);
	}
	else if ($stattype == "min") {
		$query = $this->m_minQuery[$stat];
		$filter = $this->AddAvgFilter($stat);
	}
	else {
		$this->m_int->SQLErrorLog($this->m_db.": CDBBenchmark::GetStat() bad stattype: $stattype");
		return 0;
	}
		
	# The scale gives the loop count and the specific clause:
	$startcount = 0;
	if ($scale == "minute" || $scale == "hour") 
		$maxcount = 59;
	else if ($scale == "day")
		$maxcount = 23;
	else if ($scale == "week") {
		$startcount = 1;
		$maxcount = 7;
	}
	else if ($scale == "month") {
		list($year,$month,$day) = split("[- ]",$this->m_startdate);
		$maxcount = $this->DayPerMonth($month,$year);
		$startcount = 1;
	}
	else if ($scale == "year") {
		$startcount = 1;
		$maxcount = 12;
	}
	else {
		$this->m_int->SQLErrorLog($this->m_db.": CDBBenchmark::GetStat() bad scale: $scale");
		return 0;
	}
	
	# Loop on the interval:
	for ( $idx = $startcount ; $idx <= $maxcount ; $idx++ ) {

		$q = $query;
		
		if ($scale == "minute") 
			$q .= " WHERE ".$this->m_int->GetSecondPart("LOG_DATE")." = $idx";
		else if ($scale == "hour")
			$q .= " WHERE ".$this->m_int->GetMinutePart("LOG_DATE")." = $idx";
		else if ($scale == "day")
			$q .= " WHERE ".$this->m_int->GetHourPart("LOG_DATE")." = $idx";
		else if ($scale == "week")
			$q .= " WHERE ".$this->m_int->GetDayOfWeek("LOG_DATE")." = $idx";
		else if ($scale == "month")
			$q .= " WHERE ".$this->m_int->GetDayOfMonth("LOG_DATE")." = $idx";
		else if ($scale == "year")
			$q .= " WHERE ".$this->m_int->GetMonth("LOG_DATE")." = $idx";

		$q .= $filter;

		//echo $q;
		$res = $this->m_int->QueryAndFetchaRow($q);
		if (!$res) {
			$this->m_int->SQLErrorLog($this->m_db.": CDBBenchmark::GetStat($stat) failed");
			#return 0;
		}

		$result[$idx-$startcount] = $res["VALUE"];
	}

	return $result;
}

# -------------------------------------------------
# Statistics per DAY OF MONTH:
# -------------------------------------------------
function DayPerMonth($month,$year)
{
	switch ((int)$month) {
	case 1: case 3: case 5: case 7: case 8: case 10: case 12: $daycount = 31; break;
	case 4: case 6: case 9: case 11: $daycount = 30; break;
	case 2: if (((int)$year % 4) == 0 && ((int)$year % 100)) $daycount = 29; else $daycount = 28;
	}
	
	return $daycount;
}

}; // end of class

?>
