<?php
#
# ====================================================================
#
# This is the script that displays the Real-time statistics.
#
# Date: 14 Aug 2003
# Author: Jerome Delamarche (c) Copyright INODES.  2003  All rights reserved.
# Version: 2.0
#
# Input Parameters:
#	width, height
#	duration: minute|hour|day|week|month|year
#	startdate: now|adate
#	startday, startmonth, startyear, starthour, startmin
#	viewtype: tabular|graph
#	graphtype: bar|line|pie|area
#	urls: all | url...
#	//servers: all | srv0 | srv1....
#	srv0, srv1 .... (multiple servers)
#	protocols: HTTP | HTTPS
#	responsetime,cpuload,memuse,swapuse,alertcount,apacheload,averagetime
#	threshold
#	avgval, maxval, minval
#
# Output: 
#	creates the $gImgName, $gTableOutput and the $gNavigationBar,
#	also creates the SVG files
#	output "statrtoutput.inc.php" file
#
# --------------------------------------------------------------------
#
# TODO:
#
# ====================================================================
#
include_once "util.inc.php";

# -----------------------
# Initialize the Database:
# -----------------------
include_once "messages.inc.php";
include_once "dbbenchmark.inc.php";

$db = new CDBBenchmark($dbtype,$dbname,$dbserver,$dbuser,$dbpwd);

# -----------------------------------------------------
# Globalize the input parameters (except the DATA ones):
# -----------------------------------------------------
$allparms = Array ( 
			"duration" 	=> "gDuration" ,
			"viewtype"	=> "gViewType",
			"startdate"	=> "gStartDate",
			"startyear"	=> "gStartYear",
			"startmonth"	=> "gStartMonth",
			"startday"	=> "gStartDay",
			"starthour"	=> "gStartHour",
			"startmin"	=> "gStartMin",
			"graphtype"	=> "gGraphType",
			"threshold"	=> "gThreshold",
			"urls"		=> "gUrls",
			"servers"	=> "gaServer",
			"protocols"	=> "gProtocols",
			"avgval"	=> "gAvgVal",
			"minval"	=> "gMinVal",
			"maxval"	=> "gMaxVal",
			"width"		=> "gGraphWidth",
			"height"	=> "gGraphHeight",
		);

$gHREFNavigationBar = "statrt.php?";

while (list($key,$value) = each($allparms)) {
	$$value = SetParmValue($key);

	# also computes the fixed part of the URL used for the navigation bar:
	if (strstr($value,"gStart")) continue;

	$gHREFNavigationBar .= $key."=".$$value."&";
}

# Scan the params for the list of servers:
$gServers = Array();
$parm = SetParmValue("srv0");	// "all" server ?
if (!empty($parm) || $gaServer == "srv0") $gServers[] = "all";

$srvlist = $db->GetServerList();
if ($srvlist) {
	while (list($key,$value) = each($srvlist)){
		$parm = SetParmValue("srv".($key+1));
		$addr = $value["BEN_SRVADDR"];
		if ($addr == 0) continue;
		if (!empty($parm) || $gaServer == "srv".($key+1)) {
			$gServers[] = $addr;
			$gHREFNavigationBar .= ($key+1)."=yes";
		}
	}
}
//dieError("server count=".count($gServers)." ".$gServers[0]);

# ---------------------------
# Read the user's preferences:
# ---------------------------
//$gUserPref = $db->GetUserPrefs();
$gImgType = "svg";

# ----------------
# Global variables:
# ----------------
$gBgColor	= "white";
$gBarSpacing	= 6;
$gImgName	= "";
$gXLabel	= "";
$gDelta		= 0;
$gLowTimeLimit	= 0;
$gHighTimeLimit = 0;

# ----------------------------------------------------------
# Special case: no input parm means output the default image:
# ----------------------------------------------------------
if (empty($gDuration) || empty($gViewType)) {
	// empty call:
	$gNavigationBar = "";
	$gImgName["remote"] = "../images/dfltgraph.gif";
	SendHTML();
	die;
}

# -------------------------------------------------------
# Check we can display the stats with the required format:
# -------------------------------------------------------
if ($gViewType == "graph") {
	if ($gImgType == "svg")
		include_once "svggraph.inc.php";
	else dieError("imgtype parameter is invalid (should be svg)");
}
else if ($gViewType == "tabular") {
	//include "table.inc.php";
}
else dieError("viewtype parameter is invalid (should be tabular or graph)");	

# --------------------------------
# Build the list of required stats:
# --------------------------------
include_once "statdesc.inc.php";
$gStatList = Array();
while (list($key,$value) = each($gStats)) {
	$v = SetParmValue($key);
	if (!empty($v)) $gStatList[] = $key;	
}
if (count($gStatList) == 0)
	dieError("No data specified...");

if (empty($gAvgVal) && empty($gMinVal) && empty($gMaxVal))
	dieError("No stat type specified...");

# ---------------
# Set the filters:
# ---------------
if ($gStartDate == "now") {
	$enddate = getdate();
}
else {
	$enddate["year"] = $gStartYear;
	$enddate["mon"]  = $gStartMonth;
	$enddate["mday"] = $gStartDay;
	$enddate["hours"] = $gStartHour;
	$enddate["minutes"] = $gStartMin;	
}

# Adjust the enddate according to the duration:
$enddate["seconds"] = 59;
if ($gDuration != "minute") {
	$enddate["minutes"] = 59;
	if ($gDuration != "hour") {
		$enddate["hours"] = 23;
		if ($gDuration == "week") {
			// day must be next saturday
			$curtime = mktime($enddate["hours"],$enddate["minutes"],$enddate["seconds"],$enddate["mon"],$enddate["mday"],$enddate["year"]);
			$curdate = getdate($curtime);
			$enddate["mday"] += 6 - $curdate["wday"];
			$dmonth = $db->DayPerMonth($enddate["mon"],$enddate["year"]);
			if ($enddate["mday"] > $dmonth) {
				$enddate["mon"]++;
				$enddate["mday"] -= $dmonth;	// fixed on the 30 July 2003
				if ($enddate["mon"] > 12) {
					$enddate["mon"] = 1;
					$enddate["year"]++;
				}
			}
		}
		else if ($gDuration == "month") {
				// day must be last day of current month
				$daycount = $db->DayPerMonth($enddate["mon"],$enddate["year"]);
				$enddate["mday"] = $daycount;
		}
		else if ($gDuration == "year") {
			$enddate["mon"] = 12;
			$enddate["mday"] = 31;
		}
	}
}

# Make sure values are written with 2 digits:
if ($enddate["mon"] < 10) $enddate["mon"] = "0".$enddate["mon"];
if ($enddate["mday"] < 10) $enddate["mday"] = "0".$enddate["mday"];
if ($enddate["hours"] < 10) $enddate["hours"] = "0".$enddate["hours"];
if ($enddate["minutes"] < 10) $enddate["minutes"] = "0".$enddate["minutes"];
if ($enddate["seconds"] < 10) $enddate["seconds"] = "0".$enddate["seconds"];

# Set the end date filter:
$db->SetEndDate($enddate["year"]."-".$enddate["mon"]."-".$enddate["mday"]." ".$enddate["hours"].":".$enddate["minutes"].":".$enddate["seconds"]);
$gHighTimeLimit = mktime($enddate["hours"],$enddate["minutes"],$enddate["seconds"],$enddate["mon"],$enddate["mday"],$enddate["year"]);

# Compute the start date:
if ($gDuration == "minute")
	$gDelta = 60;
else if ($gDuration == "hour")
	$gDelta = 60 * 60;
else if ($gDuration == "day")
	$gDelta = 3600 * 24;
else if ($gDuration == "week")
	$gDelta = 3600 * 24 * 7;
else if ($gDuration == "month") 
	$gDelta = 3600 * 24 * $daycount;
else if ($gDuration == "year") {
	$gDelta = 3600 * 24 * 365;
	if (($enddate["year"] % 4) == 0 && ($enddate["year"] % 100) != 0)
		$gDelta += 3600 * 24;
}

$gDelta--;
$gLowTimeLimit = $gHighTimeLimit - $gDelta;
$startdate = getdate($gLowTimeLimit);
if ($startdate["mon"] < 10) $startdate["mon"] = "0".$startdate["mon"];
if ($startdate["mday"] < 10) $startdate["mday"] = "0".$startdate["mday"];
if ($startdate["hours"] < 10) $startdate["hours"] = "0".$startdate["hours"];
if ($startdate["minutes"] < 10) $startdate["minutes"] = "0".$startdate["minutes"];
if ($startdate["seconds"] < 10) $startdate["seconds"] = "0".$startdate["seconds"];

# Set the start date filter:
$db->SetStartDate($startdate["year"]."-".$startdate["mon"]."-".$startdate["mday"]." ".$startdate["hours"].":".$startdate["minutes"].":".$startdate["seconds"]);

# Set other filters:
$db->SetThreshold($gThreshold);
$db->SetRequest($gUrls);
$db->SetProtocols($gProtocols);

# ----------------------
# Compute the statistics:
# ----------------------
$gXLabel = BuildXLabel($startdate,$enddate,$gDuration);
$gYScale = 1;
$allres  = Array();
$xcount  = 1;
$xinterv = 10;
$xstart  = 0;

if ($gDuration == "minute") {
	$xcount = 60;	// 60 seconds
}
else if ($gDuration == "hour") {
	$xcount = 60;	// 60 minutes
}
else if ($gDuration == "day") {
	$xcount = 24;	// 24 jours
	$xinterv = 1;
}
else if ($gDuration == "week") {
	$xcount = 7;	// 7 days
	$xinterv = 1;
}
else if ($gDuration == "month") {
	$xcount = $db->DayPerMonth($startdate["mon"],$startdate["year"]);
	$xinterv = 1;
	$xstart = 1;
}
else if ($gDuration == "year") {
	$xcount = 12;	// 12 months
	$xinterv = 1;
}
else dieError("Unknown scale ($gDuration)");

# Compute each required statistics: this requires three loops:
# one for the server, one for the stat and one for the stat type (max,min or avg)
for ( $srv = 0 ; $srv < count($gServers) ; $srv++ ) {
	// Set the server filter:
	$db->SetServers($gServers[$srv]);
	
	for ( $s = 0 ; $s < count($gStatList) ; $s++ ) {
		if ($gAvgVal) {
			$res = $db->GetStat($gStatList[$s],"avg",$gDuration);
			Mergedata($allres,$res,$gStatList[$s],"avg",$xcount);
		}
		if ($gMinVal) {
			$res = $db->GetStat($gStatList[$s],"min",$gDuration);
			Mergedata($allres,$res,$gStatList[$s],"min",$xcount);
		}
		if ($gMaxVal) {
			$res = $db->GetStat($gStatList[$s],"max",$gDuration);
			Mergedata($allres,$res,$gStatList[$s],"max",$xcount);
		}
	}
}

DisplayAllData($allres,$xcount,$xinterv,$xstart);

# ----------------------------------------------------
# Compute the navigation bar (displyed under the graph:
# ----------------------------------------------------
$gNavigationBar = ComputeNavigationBar();

# -------------------------
# Result output:
# -------------------------
SendHTML();
die;

# =========================================================================
# Internal Functions:
# =========================================================================

# Build the X Label from the duration and the interval of time:
function BuildXLabel($sd,$ed,$dur)
{
	# always set the year, month and day:
	$label = "From: ".$sd["mday"]."-".ShortMonth($sd["mon"])."-0".($sd["year"] % 10);
	if ($dur == "hour" || $dur == "minute") {
		$label .= " ".$sd["hours"].":".$sd["minutes"].":00";
	}
	else if ($dur == "day") { $label .= " 00:00:00"; }

	$label .= " To: ".$ed["mday"]."-".ShortMonth($ed["mon"])."-0".($ed["year"] % 10);
	if ($dur == "hour" || $dur == "minute") {
		$label .= " ".$ed["hours"].":".$ed["minutes"].":59";
	}
	else if ($dur == "day") { $label .= " 23:59:59"; }

	return $label;
}

# Filter values to avoid Database errors:
function Filter($v)
{
	if ($v < 0) return 0;
	return $v;
}

# ----------------------------
# Data merging functions:
# ----------------------------
function MergeData(&$allres,&$res,$stat,$stattype,$count)
{
	global $gYScale;

	$grnu = count($allres);
	for ( $idx = 0 ; $idx < $count ; $idx++ ) {
		$data[1][$idx] = floor($res[$idx] / $gYScale);
		$allres[$grnu][$idx] = Filter($data[1][$idx]);
	}
	$allres[$grnu]["stat"] = $stat;
	$allres[$grnu]["stattype"] = $stattype;
}

# -----------------
# Display Functions:
# -----------------
function DisplayAllData(&$allres,$xcount,$xinterv,$xstart)
{
	global $gDuration,$gShortMonth,$gShortDay,$gViewType;
	
	// X values:
	if ($gDuration == "week") {
		$idx = 0;
		while (list($key,$value) = each($gShortDay["en"]))
			$data[0][$idx++] = $value;
	}
	else if ($gDuration == "year") {
		$idx = 0;
		while (list($key,$value) = each($gShortMonth["en"]))
			$data[0][$idx++] = $value;
	}
	else {
		for ( $idx = 0 ; $idx < $xcount ; $idx++ ) {
			if ($gViewType == "graph" && ($idx % $xinterv))
				$data[0][$idx] = "";
			else
				$data[0][$idx] = $idx+$xstart;
			}
	}

	// Y values:
	for ( $c = 0 ; $c < count($allres) ; $c++ ) {
		$data[1+$c]["stat"] = $allres[$c]["stat"];
		$data[1+$c]["stattype"] = $allres[$c]["stattype"];
		for ( $idx = 0 ; $idx < $xcount ; $idx++ ) {
			$data[1+$c][$idx] = $allres[$c][$idx];
		}
	}

	// TODO: $title not set ?
	$title = '';
	CreateOutput($title,$data);
}

# ================================
# Output functions:
# ================================
function CreateOutput($title,&$data)
{
	global $gViewType;
	
	if ($gViewType == "tabular")
		CreateTable($title,$data);
	else
		CreateGraph($title,$data);
}

# --------------------------------
# Create a table with given values:
# --------------------------------
function CreateTable($title,&$data)
{
	global $gTableOutput,$gStatList,$gStats,$gXLabel,$gServers;

	$gTableOutput .= "<p class=paramtitle align=center>".$gXLabel."</p>\n";
	$gTableOutput .= "<table border=1 width=100% cellpadding=2 cellspacing=0>";
	
	# there is one row per X value
	# there is one column per data. Stats are gather according 
	# to the data displayed
	$statcount = count($gStatList);
	$stattypes = (count($data)-1) / $statcount;
	
	# count of rows for header:
	$rowhdr = 2;
	if (count($gServers) > 1) $rowhdr++;
	
	# Row with stat name:
	$gTableOutput .= "<tr>\n";
	$gTableOutput .= "<td width=40 rowspan=$rowhdr></td>\n";
	for ( $c = 0 ; $c < $statcount ; $c++ ) {
		$gTableOutput .= "<td class=textscale colspan=$stattypes>".$gStats[$gStatList[$c]]["display"]."</td>\n";
	}
	$gTableOutput .= "</tr>\n";
	
	# For SRV stats, we must adda row for each server:
	if (count($gServers) > 1) {
		$gTableOutput .= "<tr>";
		$srvcol = (count($data)-1) / count($gServers);
		for ( $s = 0 ; $s < count($gServers) ; $s++ ) {
			$gTableOutput .= "<td class=textstdblue align=center colspan=$srvcol>";
			if ($gServers[$s] == "all")
				$gTableOutput .= "all";
			else
				$gTableOutput .= long2ip($gServers[$s]);
			$gTableOutput .= "</td>\n";
		}
		$gTableOutput .= "</tr>";
	}
	
	# Row with stat types:
	$gTableOutput .= "<tr>\n";
	for ( $c = 0 ; $c < count($data)-1 ; $c++ ) {
		$gTableOutput .= "<td class=textstdbold align=center>".$data[$c+1]["stattype"]."</td>\n";
	}
	$gTableOutput .= "</tr>\n";
	
	for ( $r = 0 ; $r < count($data[0]) ; $r++ ) {
		$gTableOutput .= "<tr>\n";
		$gTableOutput .= "<td class=textstdblue>".$data[0][$r]."</td>\n";
		
		for ( $c = 1 ; $c < count($data) ; $c++ ) {
			$gTableOutput .= "<td class=textstd align=center>".$data[$c][$r]."</td>\n";
		}

		$gTableOutput .= "</tr>\n";
	}

	$gTableOutput .= "</table>\n";
}

# --------------------------------
# Create a Graph with given values:
# --------------------------------
function CreateGraph($title,&$data)
{
	global $gImgType;
	
	if ($gImgType == "svg")
		CreateSVGGraph($title,&$data);
		
	// other types not supported for now !
}

# --------------------------------
# Create a SVG Graph
# --------------------------------
function CreateSVGGraph($title,&$data)
{
	global $gGraphType,$gBarSpacing,$gGraphWidth,$gGraphHeight,$gImgName;
	global $gXLabel,$gStats,$gServers,$gProtocols,$gUrls,$gServersColor;

	$graph = new CSVGraph;
	$graph->graphicWidth      = $gGraphWidth;
	$graph->graphicHeight     = $gGraphHeight;
	$graph->plotOffsetX       = 150;
	$graph->plotOffsetY       = 20;
	$graph->plotWidth         = $graph->graphicWidth - $graph->plotOffsetX - 10;
	$graph->plotHeight        = $graph->graphicHeight - $graph->plotOffsetY - 20;
	$graph->numGridLinesX	  = 6;
	$graph->numTicksX		  = 10;
	$graph->numGridlinesY     = 11;
	$graph->numTicksY         = 1;
	$graph->innerPaddingX     = 10;
	$graph->innerPaddingY     = 10;
	$graph->outerPaddingX     = -60;
	$graph->outerPaddingY     = $graph->plotOffsetX - 50;
	$graph->offsetGridlinesX  = 0.1;
	$graph->decimalPlacesY    = 0;
	$graph->rotTagsX          = -30;
	$graph->rotTagsY          = 0;
	$graph->minY			  = 0;
	$graph->maxY			  = 100;	// minimum for maxY !
	$graph->resolutionY		  = 0;
 
	$graph->title             = $gXLabel;
	$graph->outerPadding	  = 0;
	$graph->styleTitle        = 'font-family: sans-serif; font-size: 8pt;';
	$graph->labelX            = ''; //$gXLabel;
	$graph->styleLabelX       = 'font-family: sans-serif; font-size: 6pt;';
	$graph->labelY            = "use in %    -    time in milli-sec";
	$graph->styleLabelY       = 'font-family: sans-serif; font-size: 8pt;';

	$graph->dataX			  = $data[0];
	
	# Legend Management:
	$graph->legendPosX		  = 4;
	$graph->legendPosY		  = 10;
	$graph->legendTitle       = "Legend";
	$graph->legendStyle	      = "font-family: sans-serif; font-size: 8pt;";
	$graph->legendValueStyle  = "font-family: sans-serif; font-size: 6pt;";
	
	if (count($gServers) > 1) {
		$graph->legendFilter[0]["Filtername"] = $gStats[$data[1]["stat"]]["display"];
	}
	else {
		$graph->legendFilter[0]["Filtername"] = "Server";
		$graph->legendFilter[0]["Value"] 	  = ($gServers[0] == "all") ? "all" : long2ip($gServers[0]);
	}
	$graph->legendFilter[1]["Filtername"] 	= "Protocol";
	$graph->legendFilter[1]["Value"]      	= $gProtocols;
	$graph->legendFilter[2]["Filtername"] 	= "URL";
	$graph->legendFilter[2]["Value"] 	  	= substr($gUrls,0,16);
	
	// Associate a color and a shape to each data set:
	if (count($gServers) > 1) {
		// Multiple server management:
		$datapersrv = (count($data)-1) / count($gServers);
		for ( $srvidx = 0 ; $srvidx < count($gServers) ; $srvidx++ ) {
			for ( $idx = 1+($srvidx*$datapersrv) ; $idx < 1+(($srvidx+1)*$datapersrv) ; $idx++ ) {
				$theStat = $data[$idx]["stat"];
				$graph->dataY[$idx]   = $data[$idx];
				$graph->format[$idx]  = array();
		
				// stroke type depends on the stattype:
				$stattype = $data[$idx]["stattype"];
				$stroketype = "";
				if ($stattype == "min")
					$stroketype = "stroke-dasharray: 2,2;";
				else if ($stattype == "max")
					$stroketype = "stroke-dasharray: 6,2;";
		
				$srvcolor = $gServersColor[$srvidx % count($gServersColor)];
				$graph->format[$idx]["style"] = "stroke: $srvcolor; stroke-width:1; $stroketype filter:url(#dropShadow); shape-rendering: geometricPrecision; fill:none";
			}

			$legnu = count($graph->legendValue);
			$graph->legendValue[$legnu]["text"] = ($gServers[$srvidx] == "all") ? "all" : long2ip($gServers[$srvidx]);
			$graph->legendValue[$legnu]["style"] = "stroke: $srvcolor; stroke-width:2; shape-rendering: geometricPrecision; fill:none";;
		}
	}
	else {
		// Multiple Data Management:
		$legend = Array();
		for ( $idx = 1 ; $idx < count($data) ; $idx++ ) {
			$theStat = $data[$idx]["stat"];
			$graph->dataY[$idx]   = $data[$idx];
			$graph->format[$idx]  = array();
		
			// stroke type depends on the stattype:
			$stattype = $data[$idx]["stattype"];
			$stroketype = "";
			if ($stattype == "min")
				$stroketype = "stroke-dasharray: 2,2;";
			else if ($stattype == "max")
				$stroketype = "stroke-dasharray: 6,2;";
		
			$graph->format[$idx]["style"] = "stroke:".$gStats[$theStat]["color"]."; stroke-width:1; $stroketype filter:url(#dropShadow); shape-rendering: geometricPrecision; fill:none";
			//$graph->format[$idx]["attributes"] = "marker-end='url(".$gStats[$theStat]["shape"].")'";
		
			if (empty($legend[$theStat])) {
				$legend[$theStat] = "filled";
				$legnu = count($graph->legendValue);
				$graph->legendValue[$legnu]["text"] = $gStats[$theStat]["display"];
				$graph->legendValue[$legnu]["style"] = "stroke:".$gStats[$theStat]["color"]."; stroke-width:2; shape-rendering: geometricPrecision; fill:none";;
			}
		}
	}
	
	$graph->styleTagsX    = 'font-family: sans-serif; font-size: 6pt;';
	$graph->styleTagsY    = 'font-family: sans-serif; font-size: 6pt;';
  	$graph->styleGridX	  = 'shape-rendering: crispEdges; stroke:#ccc; stroke-width:1; ';
  	$graph->styleGridY    = 'shape-rendering: crispEdges; stroke:#ccc; stroke-width:1; ';
  	$graph->styleBox      = 'shape-rendering: crispEdges; stroke:#000; stroke-width:1; fill:none;';

	// extra code for markers
	/*
	$graph->extraSVG = '
<defs>
  <marker id="square" style="stroke:#000; stroke-width:0; fill:#F00; "
    viewBox="0 0 10 10" refX="5" refY="5" orient="0"
    markerUnits="strokeWidth" markerWidth="4" markerHeight="4">
    <rect x="0" y="0" width="10" height="10"/>
  </marker>
  <marker id="circle" style="stroke:#000; stroke-width:0; fill:#0F0; "
    viewBox="0 0 10 10" refX="5" refY="5" orient="0"
    markerUnits="strokeWidth" markerWidth="5" markerHeight="5">
    <circle cx="5" cy="5" r="4"/>
  </marker>
  <marker id="triangle" style="stroke:#000; stroke-width:0; fill:#00F; "
    viewBox="0 0 10 10" refX="5" refY="5" orient="-90"
    markerUnits="strokeWidth" markerWidth="5" markerHeight="5">
    <path d="M 2 0 L 10 5 L 2 10 z" />
  </marker>
</defs>
<filter id="dropShadow" filterUnits="objectBoundingBox" x="-10%" y="-10%" width="130%" height="130%">
  <feGaussianBlur in="SourceAlpha" stdDeviation="2" result="blur"/>
  <feOffset in="blur" dx="2" dy="2" result="offsetBlur"/>
  <feMerge>
    <feMergeNode in="offsetBlur"/>
    <feMergeNode in="SourceGraphic"/>
  </feMerge>
</filter>
';
* */
	$graph->extraSVG = '
<filter id="dropShadow" filterUnits="objectBoundingBox" x="-10%" y="-10%" width="130%" height="130%">
  <feGaussianBlur in="SourceAlpha" stdDeviation="2" result="blur"/>
  <feOffset in="blur" dx="2" dy="2" result="offsetBlur"/>
  <feMerge>
    <feMergeNode in="offsetBlur"/>
    <feMergeNode in="SourceGraphic"/>
  </feMerge>
</filter>
';

	$graph->init()         or dieError($graph->error);
	$graph->drawGraph();
	
	for ( $idx = 1 ; $idx < count($data) ; $idx++ ) {
		if ($gGraphType == "bar")
			$graph->bar($idx)  or dieError($graph->error);
		else
			$graph->polyline($idx)  or dieError($graph->error);
	}

	$graph->generateSVG();

	$gImgName = GetImageName("svg");
	$fd = fopen($gImgName["local"],"w");
	fwrite($fd,$graph->svg);
	fclose($fd);
}

# -------------------------------------------
# Compute the navigation bar:
# based on start/end date + duration value...
# -------------------------------------------
function ComputeNavigationBar()
{
	global $gLowTimeLimit,$gHighTimeLimit, $gDelta,$gDuration;
	global $gHREFNavigationBar,$gStatList;

	$bar = "";
	$href = $gHREFNavigationBar."startdate=adate&";

	// Do we have a "previous" interval ?
	$t = $gLowTimeLimit;
	if ($t > 0) {
		$d = getdate($t - 1);
		$href .= "startyear=".$d["year"]."&startmonth=".$d["mon"]."&startday=".$d["mday"];
		$href .= "&starthour=".$d["hours"]."&startmin=".$d["minutes"];
		for ( $s = 0 ; $s < count($gStatList) ; $s++ ) 
			$href .= "&".$gStatList[$s]."=yes";
		$bar  .= "<a href=\"$href\"><img src=../images/prev.gif height=10 width=10 border=0></a> ";
	}

	$bar .= "1 ".$gDuration;
	$href = $gHREFNavigationBar."startdate=adate&";

	// Do we have a "next" interval ?
	$t = $gHighTimeLimit + 1;
	if ($t < time() + 1) {
		$d = getdate($t);
		$href .= "startyear=".$d["year"]."&startmonth=".$d["mon"]."&startday=".$d["mday"];
		$href .= "&starthour=".$d["hours"]."&startmin=".$d["minutes"];
		for ( $s = 0 ; $s < count($gStatList) ; $s++ ) 
			$href .= "&".$gStatList[$s]."=yes";
		$bar .= " <a href=\"$href\"><img src=../images/next.gif height=10 width=10 border=0></a>";
	}

	return $bar;
}

# --------------------------------------------
# Return the common HTML for all the statistic:
# --------------------------------------------
function SendHTML()
{
	# Clean the stats directory:
	global $gStatMaxAge;
	CleanReports($gStatMaxAge);
	
	include "statrtoutput.inc.php";
}

function dieError($text)
{
?>
<html>
	<head>
		<link rel="stylesheet" type="text/css" href="../cpanel.css">
	</head>
	<body>
		<span class=textscale>
			<?php echo $text ?>
		</span>
	</body>
</html>
<?php
	die;
}
?>
