<?php
#
# ====================================================================
#
# Base class for SVG graph output
#
# Date: 14 Aug 2003
# Author: Jerome Delamarche (c) Copyright INODES.  2003  All rights reserved.
# Version: 2.0
#
# --------------------------------------------------------------------
#
# TODO:
#
# ====================================================================
#
class CSVGraph {
	// the graph contains a plot area which has an offset from the origin:
    var $graphicWidth          	= 0;
    var $graphicHeight         	= 0;
    var $plotWidth             	= 0;
    var $plotHeight            	= 0;
    var $plotOffsetX           	= 0;
    var $plotOffsetY           	= 0;
	
	// different padding between borders and text:
    var $outerPadding          	= 0;
    var $outerPaddingX         	= 0;
    var $outerPaddingY         	= 0;
    var $innerPaddingX         	= 0;
    var $innerPaddingY         	= 0;

	// X values:
    var $dataX                 	= array();
	// Y values (2 dimension array)
    var $dataY                 	= array();

	// textual values:
    var $title                 	= '';
    var $styleTitle            	= '';
    var $styleTitleDefault     	= '';
    var $labelX                	= '';
    var $styleLabelX           	= '';
    var $styleLabelXDefault    	= '';
    var $labelY                	= '';
    var $styleLabelY           	= '';
    var $styleLabelYDefault    	= '';

    var $offsetGridlinesX      	= 0;
    var $offsetGridlinesY      	= 0;

	// minY and maxY are used as "filters" to avoid "strange" values:
    var $minY                  	= 0;
    var $maxY                  	= 0;

    var $resolutionY           	= 0;
    var $decimalPlacesY        	= 0;
    var $numGridlinesX         	= 0;
    var $numGridlinesY         	= 0;
	var $numTicksX			   	= 1;
	var $numTicksY			   	= 1;
    var $styleGridX            	= '';
    var $styleGridXDefault     	= '';
    var $styleGridY            	= '';
    var $styleGridYDefault     	= '';

	// Presentation attributes:
    var $styleBox              	= '';
    var $styleBoxDefault       	= '';
    var $styleTagsX            	= '';
    var $styleTagsXDefault     	= '';
    var $rotTagsX              	= 0;
    var $styleTagsY            	= '';
    var $styleTagsYDefault     	= '';
    var $rotTagsY              	= 0;
    var $styleLineDefault      	= '';
    var $styleBarDefault       	= '';
    var $stylePolylineDefault	= '';
    var $extraSVG              	= '';
	
	var $legendTitle		   	= '';
	var $legendPosX			   	= 0;
	var $legendPosY			   	= 0;
	var $legendStyle		   	= '';
	var $legendValue		    = array();
	var $legendValueStyle		= '';
	var $legendText				= '';
	var $legendFilter			= array(); // array of array("FilterName","Value");

	// XML result !
    var $svg                   	= '';

	// Last error message:
    var $error                 	= '';

	var $DECIMAL_POINT			= ".";
	var $THOUSANDS_SEPARATOR	= "";

	# ------------------
	# C-tor:
	# ------------------
	function svgGraph() 
	{
 	}

	# ------------------
	# Real init function:
	# ------------------
	function init() 
	{
		$graphEmpty = 0;
		
	    $this->svgText   = ''; // SVG for outer text labels and title
	    $this->svgPlot   = ''; // SVG for plot including tag text
	    $this->numGridlinesX = sizeof($this->dataX);
	    if ($this->numGridlinesX == 0) {
      		$this->error = 'No data to plot.';
      		return FALSE;
    	}
    	if ($this->numGridlinesY == 0) {
      		$this->error = 'No range for Y values. Check number of Y ticks.';
      		return FALSE;
    	}
		
    	$width = $this->plotWidth - 1;
    	$height = $this->plotHeight - 1;

     	// find range of all y values
    	$allDataY = array();
    	foreach ($this->dataY as $i => $dataY) $allDataY = array_merge($allDataY, $dataY);
    	$this->allDataY = $allDataY;
    	$data = $this->_findRange($allDataY, $this->minY, $this->maxY, $this->resolutionY);
    	$this->dataMinY = $data['min'];
    	$this->dataMaxY = $data['max'];
    	if (($this->dataMaxY - $this->dataMinY) == 0) {
      		//$this->error = 'No data to plot.';
      		//return FALSE; // data error
			$graphEmpty = 1;
    	}
    	
		// find spacing for ticks and grid lines
    	$this->deltaTicksX = $width / ($this->numGridlinesX - 1 + (2 * $this->offsetGridlinesX));
    	$this->deltaTicksY = $height / ($this->numGridlinesY - 1 + (2 * $this->offsetGridlinesY));
    	$this->factorY = ($graphEmpty) ? 1: $height / ($this->dataMaxY - $this->dataMinY);

    	// format text for tags on Y axis
    	$deltaTagsY = ($this->dataMaxY -  $this->dataMinY) / ($this->numGridlinesY - 1 + (2 * $this->offsetGridlinesY));
		if ($deltaTagsY == 0)
			$deltaTagsY = 20;	// empty graph !
    	for ($i=0; $i<$this->numGridlinesY; $i++) {
      		$text = $this->dataMinY + $deltaTagsY * ($i +  $this->offsetGridlinesY);
      		$this->tagsY[$i] = number_format($text, $this->decimalPlacesY, $this->DECIMAL_POINT, $this->THOUSANDS_SEPARATOR);
    	}
		
    	$this->tagsY = array_reverse($this->tagsY);
    	if (empty($this->tagsY)) {
      		$this->error = 'No Y axis data.';
      		return FALSE;
    	}
		
    	return TRUE;
  	}

	# ------------------------------------------------------
	# Draw the graph by computing the XML for each component:
	# ------------------------------------------------------
  	function drawGraph() 
	{
    	$this->drawOuterText();
    	$this->drawGridX();
    	$this->drawGridY();
    	$this->drawBox();
    	$this->drawTagsX();
    	$this->drawTagsY();
		$this->drawLegend();
  	}

	# ------------------------------
	# Draw the title and axis labels:
	# ------------------------------
  	function drawOuterText() 
	{
    	// draw title
    	if (!empty($this->title)) { // check if there is something to draw
      		$offset         = $this->plotOffsetX + $this->plotWidth / 2;
      		$transform      = "transform='translate(0, $this->outerPadding)'";
      		$this->svgText .= "<text text-anchor='middle' $transform x='$offset' y='1em' ";
      		$this->svgText .= "style='{$this->styleTitleDefault}{$this->styleTitle}'>";
      		$this->svgText .= "{$this->title}</text>\n";
    	}

    	//draw label Y
    	if (!empty($this->labelY)) { // check if there is something to draw
      		$offset         = $this->plotOffsetY + ($this->plotHeight / 2);
      		$transform      = "transform='translate($this->outerPaddingY, 0) rotate(-90 0 $offset)'";
      		$this->svgText .= "<text text-anchor='middle' $transform x='0' y='$offset' dy='1em' ";
      		$this->svgText .= "style='{$this->styleLabelYDefault}{$this->styleLabelY}'>";
      		$this->svgText .= "{$this->labelY}</text>\n";
    	}

    	//draw label X
    	if (!empty($this->labelX)) { // check if there is something to draw
      		$offset         = $this->plotOffsetX + ($this->plotWidth / 2);
      		$transform      = "transform='translate(0, -$this->outerPaddingX)'";
      		$this->svgText .= "<text text-anchor='middle' $transform x='$offset' y='{$this->graphicHeight}' ";
      		$this->svgText .= "style='{$this->styleLabelXDefault}{$this->styleLabelX}'>";
      		$this->svgText .= "{$this->labelX}</text>\n";
    	}
  	}

	# ---------------------
	# Draw the X grid lines:
	# ---------------------
  	function drawGridX() 
	{
    	$this->svgPlot .= "<g style='{$this->styleGridXDefault}{$this->styleGridX}'>\n";
    	$top = 0;
    	$bottom = $this->plotHeight - 1;
    	
		foreach ($this->dataX as $i => $x) {
			if ($x != "") {
	      		$u = $this->deltaTicksX * ($i +  $this->offsetGridlinesX);
	      		$this->svgPlot .= "<line x1='$u' y1='$top' x2='$u' y2='$bottom'/>\n";
			}
    	}
		
    	$this->svgPlot .= "</g>\n";
  	}

	# ---------------------
	# Draw the Y grid lines:
	# ---------------------
  	function drawGridY() 
	{
    	$this->svgPlot .= "<g style='{$this->styleGridYDefault}{$this->styleGridX}'>\n";
    	$left = 0;
    	$right = $this->plotWidth - 1;
    	
		for ($i = 0; $i < $this->numGridlinesY; $i++) {
      		$v = $this->deltaTicksY * ($i +  $this->offsetGridlinesY);
      		$this->svgPlot .= "<line x1='$left' y1='$v' x2='$right' y2='$v'/>\n";
    	}
    
		$this->svgPlot .= "</g>\n";
  	}

	# -------------------------------------
	# Draw the box around the plotting area:
	# -------------------------------------
  	function drawBox() 
	{
    	$width = $this->plotWidth - 1;
  	  	$height = $this->plotHeight - 1;
  	  	$this->svgPlot .= "<g style='{$this->styleBoxDefault}{$this->styleBox}'>\n";
 	   	$this->svgPlot .= "<polyline points='0,0 $width,0 $width,$height 0,$height 0,0'></polyline>\n";
    	$this->svgPlot .= "</g>\n";
  	}

	# ---------------------------------------------
	# Draw the X axis tag outside the plotting area:
	# --------------------------------------------
  	function drawTagsX() 
	{
    	$this->svgPlot .= "<g style='{$this->styleTagsXDefault}{$this->styleTagsX}'>\n";
    	$bottom = $this->plotHeight - 1;
    	$innerPadding = $this->innerPaddingX;

    	foreach ($this->dataX as $i => $text) {
			if ($text == "")
				continue;
				
      		$u = $this->deltaTicksX * ($i +  $this->offsetGridlinesX);

      		if ($this->rotTagsX == 0) {
        		$transform = "transform='translate(0, $innerPadding)'";
        		$this->svgPlot .= "<text text-anchor='middle' $transform dy='1em' x='$u' y='$bottom'>$text</text>\n";
      		} else if($this->rotTagsX > 0) {
        		$transform = "transform='translate(0, $innerPadding) rotate({$this->rotTagsX} $u $bottom)'";
        		$this->svgPlot .= "<text text-anchor='start' $transform x='$u' y='$bottom'>$text</text>\n";
      		} else {
        		$transform = "transform='translate(0, $innerPadding) rotate({$this->rotTagsX} $u $bottom)'";
        		$this->svgPlot .= "<text text-anchor='end' $transform x='$u' y='$bottom'>$text</text>\n";
      		}
    	}
    	
		$this->svgPlot .= "</g>\n";
  }

	# ---------------------------------------------
	# Draw the Y axis tag outside the plotting area:
	# --------------------------------------------
  	function drawTagsY() 
	{
    	if (empty($this->tagsY)) 
			return; // no data to plot. error should be picked up by init method.

    	$this->svgPlot .= "<g style='{$this->styleTagsYDefault}{$this->styleTagsY}'>\n";
    	$innerPadding = $this->innerPaddingY;
    	
		foreach ($this->tagsY as $i => $text) {
      		$v = $this->deltaTicksY * ($i +  $this->offsetGridlinesY);
      		if ($this->rotTagsY == 0) {
        		$transform = "transform='translate(-$innerPadding, 0)'";
        		$this->svgPlot .= "<text text-anchor='end' $transform x='0' dy='0.5em' y='$v'>$text</text>\n";
      		} else {
        		$transform = "transform='translate(-$innerPadding, 0) rotate({$this->rotTagsY} 0 $v)'";
        		$this->svgPlot .= "<text text-anchor='end' $transform x='0' dy='0.5em' y='$v'>$text</text>\n";
      		}
    	}
    
		$this->svgPlot .= "</g>\n";
  	}
	
	# ---------------
	# Draw the legend:
	# ---------------
	function drawLegend()
	{	
		// compute the legend vertical position:
		$markerHeight = 44;
		$lposX = 0;
		$lposY = ($this->graphicHeight - (count($this->legendValue) * 13 + 40 + $markerHeight)) / 2;
		$lwidth = 76;
		$lheight = $lposY + ((count($this->legendValue)+1) * 13 + $markerHeight);
		
		// display the title
		$this->legendText .= "<text style='{$this->legendStyle}' x='$lposX' y='$lposY'>{$this->legendTitle}</text>\n";
    	$this->legendText .= "<g style='{$this->legendValueStyle}'>\n";
		
		// surround the legend with a box:
		$lposY += 8;
		$this->legendText .= "<polyline style='{$this->styleBoxDefault}{$this->styleBox} filter:url(#dropShadow);' ".
								"points='$lposX,$lposY $lposX,$lheight, $lwidth,$lheight $lwidth,$lposY $lposX,$lposY'></polyline>";

		// display the legend info:
		$lposX += 3;
		foreach ($this->legendValue as $i => $value) {
			$x2 = $lposX + 10;
			$ypos = $lposY + 12 + $i * 13;
			$this->legendText .= "<line style='{$value["style"]}' x1='$lposX' y1='$ypos' x2='$x2' y2='$ypos'></line>";
			$ypos += 3;
			$x2 += 2;
			$this->legendText .= "<text x='$x2' y='$ypos'>{$value["text"]}</text>\n";
		}
		
		// Display the marker info:
		$ypos += 7;
		$x2 = $lwidth - 6;
		$this->legendText .= "<line x1='$lposX' y1='$ypos' x2='$x2' y2='$ypos'></line>";
		$lposY += 20 + count($this->legendValue) * 13;
		$stroketype = Array( "", "stroke-dasharray: 2,2;", "stroke-dasharray: 6,2;");
		$markertext = Array( "Average", "Min", "Max" );
		for ( $m = 0 ; $m < 3 ; $m++ ) {
			$ypos = $lposY + $m * 10;
			$x2 = $lposX + 10;
			$this->legendText .= "<line style='stroke: black; ".$stroketype[$m]."stroke-width:2; shape-rendering: geometricPrecision; fill:none' x1='$lposX' y1='$ypos' x2='$x2' y2='$ypos'></line>";
			$ypos += 3;
			$x2 += 2;
			$this->legendText .= "<text x='$x2' y='$ypos'>".$markertext[$m]."</text>\n";
		}
		
		// Display the filters:
		$yposval = $lheight + 14;
		$xposval = 35;
		foreach ($this->legendFilter as $i => $value) {
			$this->legendText .= "<text x='0' y='$yposval'>{$value["Filtername"]}:</text>\n";
			$this->legendText .= "<text x='$xposval' y='$yposval'>{$value["Value"]}</text>\n";
			$yposval += 12;
		}
		
		$this->legendText .= "</g>\n";
	}

	# ------------------------------------------
	# Draw a line from point to point.
	# Draw the shape at each intersection.
	# ------------------------------------------
  	function polyLine($whichDataSet) 
	{
    	if (empty($this->format[$whichDataSet]['style'])) {
      		$this->error = 'No style defined for data plot.';
      		return FALSE; // data error
    	}
    	
		$attributes = empty($this->format[$whichDataSet]['attributes']) ?
                  '' : $this->format[$whichDataSet]['attributes'];

    	$this->svgPlot .= "<g style='{$this->stylePolylineDefault}{$this->format[$whichDataSet]['style']}'>\n";
    	$this->svgPlot .= "<polyline $attributes ";
    	$u = 0;
    	$v = 0;
    	
		foreach ($this->dataX as $i => $x) {
      		$y = $this->dataY[$whichDataSet][$i];
      		$u = $this->deltaTicksX * ($i +  $this->offsetGridlinesX);
      		$v = $this->factorY * ($y - $this->dataMinY);
      		$v = $this->plotHeight - $v;
      		if ($i==0) $this->svgPlot .= "points='$u,$v";
      		else  $this->svgPlot .= " $u,$v ";
      		$oldU = $u;
      		$oldV = $v;
    	}
    
		$this->svgPlot .= "'/>\n</g>\n";
    
		return TRUE;
  	}

	# ------------------------------------------
	# Draw a line from point to point.
	# Draw the shape at each intersection.
	# ------------------------------------------
  	function line($whichDataSet) 
  	{
    	if (empty($this->format[$whichDataSet]['style'])) {
      		$this->error = "No style defined for data plot ($whichDataSet). Check parameters.";
      		return FALSE; // data error
    	}

    	$attributes = empty($this->format[$whichDataSet]['attributes']) ?
                  '' : $this->format[$whichDataSet]['attributes'];
    	$this->svgPlot .= "<g style='{$this->styleLineDefault}{$this->format[$whichDataSet]['style']}'>\n";
    	$u = 0;
    	$v = 0;
    	
		foreach ($this->dataX as $i => $x) {
      		$y = $this->dataY[$whichDataSet][$i];
      		$u = $this->deltaTicksX * ($i +  $this->offsetGridlinesX);
      		$v = $this->factorY * ($y - $this->dataMinY);
      		$v = $this->plotHeight - $v;
      		if ($i==0) {
        		$oldU = $u;
        		$oldV = $v;
      		}
      		$this->svgPlot .= "<line $attributes x1='$oldU' y1='$oldV' x2='$u' y2='$v'/>\n";
      		$oldU = $u;
      		$oldV = $v;
    	}
		$this->svgPlot .= "</g>\n";
    	
		return TRUE;
  	}

	# -----------------------------------
	# Draw a bar for each data of the set:
	# -----------------------------------
  	function bar($whichDataSet) 
	{
    	if (!isset($this->format[$whichDataSet]['style'])    ||
        	!isset($this->format[$whichDataSet]['barWidth']) ||
        	!isset($this->format[$whichDataSet]['barOffset']  )) {
      		$this->error = 'Style parameters missing for bar plot.';
      		return FALSE; // data error
    	}
    	
		$this->svgPlot .= "<g style='{$this->styleBarDefault}{$this->format[$whichDataSet]['style']}'>\n";
    	$barWidth = $this->format[$whichDataSet]['barWidth'] * $this->deltaTicksX;
    	$barOffset = $this->format[$whichDataSet]['barOffset'] * $barWidth;
    	$u = 0;
    	$v = 0;
    
		foreach ($this->dataX as $i => $x) {
      		$y = $this->dataY[$whichDataSet][$i];
      		$u = $this->deltaTicksX * ($i + $this->offsetGridlinesX) - ($barWidth / 2) + $barOffset;
      		$v = $this->factorY * ($y - $this->dataMinY);
      		$v = $this->plotHeight - $v;
      		$height = $this->factorY * ($y - $this->dataMinY) - 1;

      		$this->svgPlot .= "<rect x='$u' y='$v' width='$barWidth' height='$height'/>\n";
    	}
    	$this->svgPlot .= "</g>\n";
    
		return TRUE;
  	}

  	# ---------------------------------------------------------------------------
  	# Determine the range of values for a dataset
  	# ---------------------------------------------------------------------------
  	function _findRange($data, $min, $max, $resolution) 
	{
    	if (sizeof($data) == 0 ) return array('min' => 0, 'max' => 0);
	
	    foreach ($data as $key => $value) {
      		if ($value=='none') continue;
      		if ($value > $max) $max = $value;
      		if ($value < $min) $min = $value;
    	}
    	if ($max == 0) {
      		$factor = 1;
    	} else {
      		if ($max < 0) $factor = - pow(10, (floor(log10(abs($max))) + $resolution) );
      		else $factor = pow(10, (floor(log10(abs($max))) - $resolution) );
    	}
    	$max = $factor * @ceil($max / $factor);
    	$min = $factor * @floor($min / $factor);

    	return array('min' => $min, 'max' => $max);
  	}

	# -----------------------------------------
  	# OK: Generate the XML for the entire graph:
	# -----------------------------------------
  	function generateSVG() 
	{
    	$this->svg  = "<?xml version='1.0' encoding='iso-8859-1'?>\n";
	    $this->svg .= "<!DOCTYPE svg PUBLIC '-//W3C//DTD SVG 20000303 Stylable//EN' ";
	    $this->svg .= "'http://www.w3.org/TR/2000/03/WD-SVG-20000303/DTD/svg-20000303-stylable.dtd'>\n";
	    $this->svg .= "<svg width='{$this->graphicWidth}' height='{$this->graphicHeight}'>\n";

	    if (!empty($this->extraSVG)) {
    		$this->svg .= "<!-- Extra SVG -->\n";
      		$this->svg .= $this->extraSVG."\n";
      		$this->svg .= "<!-- End Extra SVG -->\n";
    	}

    	if (!empty($this->svgText)) {
      		$this->svg .= "<!-- Outer Text -->\n";
      		$this->svg .= "<g>\n";
      		$this->svg .= $this->svgText."\n";
      		$this->svg .= "</g>\n" ;
      		$this->svg .= "<!-- End Outer Text -->\n";
    	}
		
		if (!empty($this->legendText)) {
      		$this->svg .= "<!-- Start Legend -->\n";
      		$this->svg .= $this->legendText."\n";
      		$this->svg .= "<!-- End Legend -->\n";
		}

    	$this->svg .= "<!-- Plot Area -->\n";
    	$this->svg .= "<g transform='translate({$this->plotOffsetX},{$this->plotOffsetY})'>\n";
    	$this->svg .= $this->svgPlot."\n";
    	$this->svg .= "</g>\n" ;
    	$this->svg .= "<!-- End Plot Area -->\n";

    	$this->svg .= "</svg>\n" ;
  	}
};
?>
